import{u as n,a as p,I as m,r as f,s as u,j as a}from"./index-CE-9RlbR.js";import{F as d}from"./startCase-BwzMWzGM.js";import{F as c}from"./FeatureWrapper-CyVtzJVN.js";import{g}from"./DisplayValue-eLtXORWe.js";import"./envs-CRUy-_iG.js";import"./isObject--vsEa_js.js";import"./_createCompounder-pwnXEtry.js";import"./isObjectLike-BsEMHB57.js";function y({sourceIdx:t,device:s}){const{t:i}=n("common"),r=p(m(e=>e.bridgeInfo[t])),o=f.useCallback(async e=>{await u(t,"bridge/request/device/options",{id:s.ieee_address,options:e})},[t,s.ieee_address]);return s.definition?.options?.length?a.jsx("div",{className:"list bg-base-100",children:s.definition.options.map(e=>a.jsx(d,{feature:e,device:s,deviceState:r.config.devices[s.ieee_address]??{},onChange:o,featureWrapperClass:c,parentFeatures:[]},g(e)))}):i(e=>e.empty_exposes_definition)}export{y as default};
