export declare const END = 192;
export declare const ESCAPE = 219;
export declare const ESCEND = 220;
export declare const ESCESC = 221;
export declare const SIGNATURE = 57005;
export declare const ZBOSS_NCP_API_HL = 6;
export declare const ZBOSS_FLAG_FIRST_FRAGMENT = 64;
export declare const ZBOSS_FLAG_LAST_FRAGMENT = 128;
//# sourceMappingURL=consts.d.ts.map