export declare enum TypeFlags {
    None = 0,
    Any = 1,
    Unknown = 2,
    Undefined = 4,
    Null = 8,
    Void = 16,
    String = 32,
    Number = 64,
    BigInt = 128,
    Boolean = 256,
    ESSymbol = 512,
    StringLiteral = 1024,
    NumberLiteral = 2048,
    BigIntLiteral = 4096,
    BooleanLiteral = 8192,
    UniqueESSymbol = 16384,
    EnumLiteral = 32768,
    Enum = 65536,
    NonPrimitive = 131072,
    Never = 262144,
    TypeParameter = 524288,
    Object = 1048576,
    Index = 2097152,
    TemplateLiteral = 4194304,
    StringMapping = 8388608,
    Substitution = 16777216,
    IndexedAccess = 33554432,
    Conditional = 67108864,
    Union = 134217728,
    Intersection = 268435456,
    Reserved1 = 536870912,
    Reserved2 = 1073741824,
    Reserved3 = -2147483648,
    AnyOrUnknown = 3,
    Nullable = 12,
    Literal = 15360,
    Unit = 97292,
    Freshable = 80896,
    StringOrNumberLiteral = 3072,
    StringOrNumberLiteralOrUnique = 19456,
    DefinitelyFalsy = 15388,
    PossiblyFalsy = 15868,
    Intrinsic = 393983,
    StringLike = 12583968,
    NumberLike = 67648,
    BigIntLike = 4224,
    BooleanLike = 8448,
    EnumLike = 98304,
    ESSymbolLike = 16896,
    VoidLike = 20,
    Primitive = 12713980,
    DefinitelyNonNullable = 13893600,
    DisjointDomains = 12812284,
    UnionOrIntersection = 402653184,
    StructuredType = 403701760,
    TypeVariable = 34078720,
    InstantiableNonPrimitive = 117964800,
    InstantiablePrimitive = 14680064,
    Instantiable = 132644864,
    StructuredOrInstantiable = 536346624,
    ObjectFlagsType = 403963917,
    Simplifiable = 102760448,
    Singleton = 394239,
    Narrowable = 536575971,
    IncludesMask = 416808959,
    IncludesMissingType = 524288,
    IncludesNonWideningType = 2097152,
    IncludesWildcard = 33554432,
    IncludesEmptyObject = 67108864,
    IncludesInstantiable = 16777216,
    IncludesConstrainedTypeVariable = 536870912,
    IncludesError = 1073741824,
    NotPrimitiveUnion = 286523411
}
//# sourceMappingURL=typeFlags.enum.d.ts.map