"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZBOSS_FLAG_LAST_FRAGMENT = exports.ZBOSS_FLAG_FIRST_FRAGMENT = exports.ZBOSS_NCP_API_HL = exports.SIGNATURE = exports.ESCESC = exports.ESCEND = exports.ESCAPE = exports.END = void 0;
exports.END = 0xc0;
exports.ESCAPE = 0xdb;
exports.ESCEND = 0xdc;
exports.ESCESC = 0xdd;
exports.SIGNATURE = 0xdead;
exports.ZBOSS_NCP_API_HL = 0x06;
exports.ZBOSS_FLAG_FIRST_FRAGMENT = 0x40;
exports.ZBOSS_FLAG_LAST_FRAGMENT = 0x80;
//# sourceMappingURL=consts.js.map