import{a,I as r,j as i}from"./index-CE-9RlbR.js";import{J as p}from"./Json-H0wmpNla.js";import"./envs-CRUy-_iG.js";function S({sourceIdx:t,friendlyName:e}){const o=a(r(s=>s.deviceStates[t][e]));return i.jsx(p,{obj:o??{}})}export{S as default};
