import{r as c,u as m,bb as k,b1 as i,j as s,L as p,F as u,bc as b}from"./index-CE-9RlbR.js";const h=c.memo(({supported:n,definitionModel:e,icon:o=!1})=>{const{t:r}=m("zigbee"),t=e||r(l=>l.unknown),a=n&&e?`https://www.zigbee2mqtt.io/devices/${encodeURIComponent(k(e))}.html`:i;return s.jsxs(p,{target:"_blank",rel:"noopener noreferrer",to:a,className:"link link-hover",children:[t,o?s.jsx(u,{icon:b,className:"ms-0.5"}):null]})}),x=c.memo(({supported:n,definitionVendor:e,icon:o=!1})=>{const{t:r}=m("zigbee"),t=e||r(l=>l.unknown),a=n&&e?`https://www.zigbee2mqtt.io/supported-devices/#v=${encodeURIComponent(e)}`:i;return s.jsxs(p,{target:"_blank",rel:"noopener noreferrer",to:a,className:"link link-hover",children:[t,o?s.jsx(u,{icon:b,className:"ms-0.5"}):null]})});export{h as D,x as V};
