const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["./About-CPBYVYHu.js","./index-CE-9RlbR.js","./envs-CRUy-_iG.js","./index-Bz12kIVH.css","./snakeCase-WwJ9QoNs.js","./_createCompounder-pwnXEtry.js","./isObjectLike-BsEMHB57.js","./Health-CGiav1nz.js","./useTable-CR-pMENX.js","./DebouncedInput-D3cHKuwF.js","./DeviceImage-Bcp15E0n.js","./index-CDoSh7QA.js","./InfoAlert-AzL4ZLp4.js","./Table-KdlIQbBf.js","./Duration-DRYmDbwS.js","./TimeAgo-D_O5nVVz.js","./Settings-DD_XQp0Y.js","./SettingsList-DXaUDCbO.js","./CheckboxField-BPvsXluL.js","./InputField-DRmYNLGX.js","./NumberField-XGSCkVuy.js","./SelectField-BPXpeItQ.js","./Tools-gr9RLnwh.js","./Bridge-ByD124CA.js","./Json-H0wmpNla.js","./DevConsole-DBxHJPsB.js","./TextareaField-Ba8vaWW5.js"])))=>i.map(i=>d[i]);
import{u as p,C as b,E as f,G as h,r as o,K as j,b as x,T,aA as E,aB as N,Z as $,j as s,J as _,F as y,N as A,M as P,A as w,S,$ as l}from"./index-CE-9RlbR.js";import"./envs-CRUy-_iG.js";const z=o.lazy(async()=>await l(()=>import("./About-CPBYVYHu.js"),__vite__mapDeps([0,1,2,3,4,5,6]),import.meta.url)),L=o.lazy(async()=>await l(()=>import("./Health-CGiav1nz.js"),__vite__mapDeps([7,1,2,3,8,9,10,11,12,13,14,15]),import.meta.url)),D=o.lazy(async()=>await l(()=>import("./Settings-DD_XQp0Y.js"),__vite__mapDeps([16,1,2,3,12,17,18,19,20,21]),import.meta.url)),R=o.lazy(async()=>await l(()=>import("./Tools-gr9RLnwh.js"),__vite__mapDeps([22,1,2,3,19,11]),import.meta.url)),V=o.lazy(async()=>await l(()=>import("./Bridge-ByD124CA.js"),__vite__mapDeps([23,1,2,3,24]),import.meta.url)),C=o.lazy(async()=>await l(()=>import("./DevConsole-DBxHJPsB.js"),__vite__mapDeps([25,1,2,3,19,21,26,12]),import.meta.url));function O(t,n,a){switch(n){case"about":return s.jsx(z,{sourceIdx:t},t);case"health":return s.jsx(L,{sourceIdx:t},t);case"settings":return s.jsx(D,{sourceIdx:t,tab:a??"main"},t);case"tools":return s.jsx(R,{sourceIdx:t},t);case"bridge":return s.jsx(V,{sourceIdx:t},t);case"dev-console":return s.jsx(C,{sourceIdx:t,tab:a??"mqtt"},t)}}const d=({isActive:t})=>t?"menu-active":void 0;function F(){const{t}=p("settings"),n=b(),{sourceIdx:a,tab:i,subTab:r}=f(),[g,c]=h(a);o.useEffect(()=>{!a||!c?n(`/settings/0/${i||"about"}`,{replace:!0}):i?r||(i==="settings"?n(`/settings/${a}/settings/main`,{replace:!0}):i==="dev-console"&&n(`/settings/${a}/dev-console/mqtt`,{replace:!0})):n(`/settings/${a}/about`,{replace:!0})},[a,c,i,r,n]);const v=o.useMemo(()=>[{to:`/settings/${a}/about`,icon:j,title:t(e=>e.about)},{to:`/settings/${a}/health`,icon:x,title:t(e=>e.health)},{to:`/settings/${a}/settings`,icon:T,title:t(e=>e.settings)},{to:`/settings/${a}/tools`,icon:E,title:t(e=>e.tools)},{to:`/settings/${a}/bridge`,icon:N,title:t(e=>e.bridge)},{to:`/settings/${a}/dev-console`,icon:$,title:t(e=>e.dev_console)}],[a,t]),m=s.jsx("div",{className:"menu menu-horizontal flex-1",children:v.map(e=>s.jsx("li",{className:"tooltip tooltip-bottom","data-tip":e.title,children:s.jsxs(_,{to:e.to,className:d,children:[s.jsx(y,{icon:e.icon,size:"lg"}),s.jsx("span",{className:"hidden lg:inline",children:e.title})]})},e.to))});return s.jsxs(s.Fragment,{children:[s.jsx(A,{children:P?s.jsxs("div",{className:"flex-1 flex flex-col",children:[s.jsx("div",{className:"menu menu-horizontal flex-1 pb-0",children:w.map((e,u)=>s.jsx("li",{children:s.jsx(_,{to:`/settings/${u}/${i||"about"}${r?`/${r}`:""}`,className:d,children:s.jsx(S,{idx:u,alwaysShowName:!0})})},e))}),m]}):m}),i&&O(g,i,r)]})}export{F as default};
