import type { Endpoint, Group } from "zigbee-herdsman/dist/controller/model";
import type { ModernExtend, Tz } from "./types";
declare const tz: {
    setModel: {
        key: string[];
        convertSet: (entity: Endpoint | Group, key: string, value: unknown, meta: Tz.Meta) => Promise<{
            state: {
                model: unknown;
            };
        }>;
    };
};
declare const extend: {
    configureReadModelID: () => ModernExtend;
    externalSwitchType: () => ModernExtend;
    minimumPWM: () => ModernExtend;
    SRZG9002KR12Pro: () => ModernExtend;
    SRZG2836D5Pro: () => ModernExtend;
    SRZG9002K16Pro: () => ModernExtend;
    indicatorLight(): ModernExtend;
    thermostatWeeklySchedule: () => ModernExtend;
    thermostatChildLock: () => ModernExtend;
    thermostatPreset: () => ModernExtend;
    thermostatCurrentHeatingSetpoint: () => ModernExtend;
    SRZG2856Pro: () => ModernExtend;
    motorControl: () => ModernExtend;
};
export { extend, tz };
//# sourceMappingURL=sunricher.d.ts.map