import events from "node:events";
import type { Adapter } from "../../adapter";
import type Database from "../database";
export declare abstract class Entity<T extends Record<keyof T, any[]> = Record<string, any[]>> extends events.EventEmitter<T> {
    protected static database: Database;
    protected static adapter: Adapter;
    static injectDatabase(database: Database): void;
    static injectAdapter(adapter: Adapter): void;
}
export default Entity;
//# sourceMappingURL=entity.d.ts.map