import{a as o,I as t,j as e}from"./index-CE-9RlbR.js";import{A as i,R as m}from"./RecallRemove-DFcPL08D.js";import"./envs-CRUy-_iG.js";import"./DashboardFeatureWrapper-BAseHpKP.js";import"./startCase-BwzMWzGM.js";import"./DisplayValue-eLtXORWe.js";import"./isObject--vsEa_js.js";import"./_createCompounder-pwnXEtry.js";import"./isObjectLike-BsEMHB57.js";import"./InputField-DRmYNLGX.js";import"./react-select.esm-CxImRSNQ.js";import"./floating-ui.dom-D02yrJtw.js";function w({sourceIdx:r,device:a}){const s=o(t(d=>d.deviceStates[r][a.friendly_name]));return e.jsxs("div",{className:"flex flex-row flex-wrap gap-4 w-full",children:[e.jsx("div",{className:"card card-border bg-base-200 border-base-300 rounded-box shadow-md flex-1",children:e.jsx("div",{className:"card-body p-4",children:e.jsx(i,{sourceIdx:r,target:a,deviceState:s??{}})})}),e.jsx("div",{className:"card card-border bg-base-200 border-base-300 rounded-box shadow-md flex-1",children:e.jsx("div",{className:"card-body p-4",children:e.jsx(m,{sourceIdx:r,target:a})})})]})}export{w as default};
