declare const generateUniqueNumber: (collection: Map<number, any> | Set<number>) => number;
declare const addUniqueNumber: (set: Set<number>) => number;
export { addUniqueNumber, generateUniqueNumber };
//# sourceMappingURL=module.d.ts.map