import type { Fz } from "../lib/types";
export declare const fan: Fz.Converter<"hvacFanCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const fan_speed: Fz.Converter<"genLevelCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const thermostat: Fz.Converter<"hvacThermostat", undefined, ["attributeReport", "readResponse"]>;
export declare const thermostat_weekly_schedule: Fz.Converter<"hvacThermostat", undefined, ["commandGetWeeklyScheduleRsp"]>;
export declare const hvac_user_interface: Fz.Converter<"hvacUserInterfaceCfg", undefined, ["attributeReport", "readResponse"]>;
export declare const lock_operation_event: Fz.Converter<"closuresDoorLock", undefined, "commandOperationEventNotification">;
export declare const lock_programming_event: Fz.Converter<"closuresDoorLock", undefined, "commandProgrammingEventNotification">;
export declare const lock_programming_event_read_pincode: Fz.Converter<"closuresDoorLock", undefined, "commandProgrammingEventNotification">;
export declare const lock: Fz.Converter<"closuresDoorLock", undefined, ["attributeReport", "readResponse"]>;
export declare const lock_set_pin_code_response: Fz.Converter<"closuresDoorLock", undefined, ["commandSetPinCodeRsp", "commandClearPinCodeRsp"]>;
export declare const lock_pin_code_response: Fz.Converter<"closuresDoorLock", undefined, ["commandGetPinCodeRsp"]>;
export declare const lock_user_status_response: Fz.Converter<"closuresDoorLock", undefined, ["commandGetUserStatusRsp"]>;
export declare const linkquality_from_basic: Fz.Converter<"genBasic", undefined, ["attributeReport", "readResponse"]>;
export declare const battery: Fz.Converter<"genPowerCfg", undefined, ["attributeReport", "readResponse"]>;
export declare const temperature: Fz.Converter<"msTemperatureMeasurement", undefined, ["attributeReport", "readResponse"]>;
export declare const device_temperature: Fz.Converter<"genDeviceTempCfg", undefined, ["attributeReport", "readResponse"]>;
export declare const humidity: Fz.Converter<"msRelativeHumidity", undefined, ["attributeReport", "readResponse"]>;
export declare const pm25: Fz.Converter<"pm25Measurement", undefined, ["attributeReport", "readResponse"]>;
export declare const flow: Fz.Converter<"msFlowMeasurement", undefined, ["attributeReport", "readResponse"]>;
export declare const soil_moisture: Fz.Converter<"msSoilMoisture", undefined, ["attributeReport", "readResponse"]>;
export declare const pressure: Fz.Converter<"msPressureMeasurement", undefined, ["attributeReport", "readResponse"]>;
export declare const co2: Fz.Converter<"msCO2", undefined, ["attributeReport", "readResponse"]>;
export declare const occupancy: Fz.Converter<"msOccupancySensing", undefined, ["attributeReport", "readResponse"]>;
export declare const occupancy_with_timeout: Fz.Converter<"msOccupancySensing", undefined, ["attributeReport", "readResponse"]>;
export declare const occupancy_timeout: Fz.Converter<"msOccupancySensing", undefined, ["attributeReport", "readResponse"]>;
export declare const brightness: Fz.Converter<"genLevelCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const level_config: Fz.Converter<"genLevelCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const color_colortemp: Fz.Converter<"lightingColorCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const meter_identification: Fz.Converter<"seMeterIdentification", undefined, ["readResponse"]>;
export declare const metering: Fz.Converter<"seMetering", undefined, ["attributeReport", "readResponse"]>;
export declare const electrical_measurement: Fz.Converter<"haElectricalMeasurement", undefined, ["attributeReport", "readResponse"]>;
export declare const gas_metering: Fz.Converter<"seMetering", undefined, ["attributeReport", "readResponse"]>;
export declare const on_off: Fz.Converter<"genOnOff", undefined, ["attributeReport", "readResponse"]>;
export declare const on_off_force_multiendpoint: Fz.Converter<"genOnOff", undefined, ["attributeReport", "readResponse"]>;
export declare const on_off_skip_duplicate_transaction: Fz.Converter<"genOnOff", undefined, ["attributeReport", "readResponse"]>;
export declare const power_on_behavior: Fz.Converter<"genOnOff", undefined, ["attributeReport", "readResponse"]>;
export declare const ias_no_alarm: Fz.Converter<"ssIasZone", undefined, ["attributeReport", "commandStatusChangeNotification"]>;
export declare const ias_siren: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_water_leak_alarm_1: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_water_leak_alarm_1_report: Fz.Converter<"ssIasZone", undefined, "attributeReport">;
export declare const ias_vibration_alarm_1: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_vibration_alarm_1_with_timeout: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_gas_alarm_1: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_gas_alarm_2: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_smoke_alarm_1: Fz.Converter<"ssIasZone", undefined, ["commandStatusChangeNotification", "attributeReport", "readResponse"]>;
export declare const ias_contact_alarm_1: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_contact_alarm_1_report: Fz.Converter<"ssIasZone", undefined, ["attributeReport", "readResponse"]>;
export declare const ias_carbon_monoxide_alarm_1: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_carbon_monoxide_alarm_1_gas_alarm_2: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_sos_alarm_2: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_occupancy_alarm_1: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_occupancy_alarm_1_report: Fz.Converter<"ssIasZone", undefined, ["attributeReport", "readResponse"]>;
export declare const ias_occupancy_alarm_2: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_alarm_only_alarm_1: Fz.Converter<"ssIasZone", undefined, "attributeReport">;
export declare const ias_occupancy_only_alarm_2: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ias_occupancy_alarm_1_with_timeout: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const command_store: Fz.Converter<"genScenes", undefined, "commandStore">;
export declare const command_recall: Fz.Converter<"genScenes", undefined, "commandRecall">;
export declare const command_panic: Fz.Converter<"ssIasAce", undefined, "commandPanic">;
export declare const command_arm: Fz.Converter<"ssIasAce", undefined, "commandArm">;
export declare const command_cover_stop: Fz.Converter<"closuresWindowCovering", undefined, "commandStop">;
export declare const command_cover_open: Fz.Converter<"closuresWindowCovering", undefined, "commandUpOpen">;
export declare const command_cover_close: Fz.Converter<"closuresWindowCovering", undefined, "commandDownClose">;
export declare const command_on: Fz.Converter<"genOnOff", undefined, "commandOn">;
export declare const command_off: Fz.Converter<"genOnOff", undefined, "commandOff">;
export declare const command_off_with_effect: Fz.Converter<"genOnOff", undefined, "commandOffWithEffect">;
export declare const command_toggle: Fz.Converter<"genOnOff", undefined, "commandToggle">;
export declare const command_move_to_level: Fz.Converter<"genLevelCtrl", undefined, ["commandMoveToLevel", "commandMoveToLevelWithOnOff"]>;
export declare const command_move: Fz.Converter<"genLevelCtrl", undefined, ["commandMove", "commandMoveWithOnOff"]>;
export declare const command_step: Fz.Converter<"genLevelCtrl", undefined, ["commandStep", "commandStepWithOnOff"]>;
export declare const command_stop: Fz.Converter<"genLevelCtrl", undefined, ["commandStop", "commandStopWithOnOff"]>;
export declare const command_move_color_temperature: Fz.Converter<"lightingColorCtrl", undefined, ["commandMoveColorTemp"]>;
export declare const command_stop_move_step: Fz.Converter<"lightingColorCtrl", undefined, "commandStopMoveStep">;
export declare const command_step_color_temperature: Fz.Converter<"lightingColorCtrl", undefined, "commandStepColorTemp">;
export declare const command_enhanced_move_to_hue_and_saturation: Fz.Converter<"lightingColorCtrl", undefined, "commandEnhancedMoveToHueAndSaturation">;
export declare const command_move_to_hue_and_saturation: Fz.Converter<"lightingColorCtrl", undefined, "commandMoveToHueAndSaturation">;
export declare const command_step_hue: Fz.Converter<"lightingColorCtrl", undefined, ["commandStepHue"]>;
export declare const command_step_saturation: Fz.Converter<"lightingColorCtrl", undefined, ["commandStepSaturation"]>;
export declare const command_color_loop_set: Fz.Converter<"lightingColorCtrl", undefined, "commandColorLoopSet">;
export declare const command_move_to_color_temp: Fz.Converter<"lightingColorCtrl", undefined, "commandMoveToColorTemp">;
export declare const command_move_to_color: Fz.Converter<"lightingColorCtrl", undefined, "commandMoveToColor">;
export declare const command_move_hue: Fz.Converter<"lightingColorCtrl", undefined, "commandMoveHue">;
export declare const command_move_to_saturation: Fz.Converter<"lightingColorCtrl", undefined, "commandMoveToSaturation">;
export declare const command_move_to_hue: Fz.Converter<"lightingColorCtrl", undefined, "commandMoveToHue">;
export declare const command_emergency: Fz.Converter<"ssIasAce", undefined, "commandEmergency">;
export declare const command_on_state: Fz.Converter<"genOnOff", undefined, "commandOn">;
export declare const command_off_state: Fz.Converter<"genOnOff", undefined, "commandOff">;
export declare const identify: Fz.Converter<"genIdentify", undefined, ["attributeReport", "readResponse"]>;
export declare const cover_position_tilt: Fz.Converter<"closuresWindowCovering", undefined, ["attributeReport", "readResponse"]>;
export declare const cover_position_via_brightness: Fz.Converter<"genLevelCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const cover_state_via_onoff: Fz.Converter<"genOnOff", undefined, ["attributeReport", "readResponse"]>;
export declare const curtain_position_analog_output: Fz.Converter<"genAnalogOutput", undefined, ["attributeReport", "readResponse"]>;
export declare const lighting_ballast_configuration: Fz.Converter<"lightingBallastCfg", undefined, ["attributeReport", "readResponse"]>;
export declare const checkin_presence: Fz.Converter<"genPollCtrl", undefined, ["commandCheckin"]>;
export declare const ias_enroll: Fz.Converter<"ssIasZone", undefined, ["attributeReport", "readResponse"]>;
export declare const power_source: Fz.Converter<"genBasic", undefined, ["attributeReport", "readResponse"]>;
export declare const hw_version: Fz.Converter<"genBasic", undefined, ["attributeReport", "readResponse"]>;
export declare const ewelink_action: Fz.Converter<"genOnOff", undefined, ["commandOn", "commandOff", "commandToggle"]>;
export declare const command_status_change_notification_action: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ignore_onoff_report: Fz.Converter<"genOnOff", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_illuminance_report: Fz.Converter<"msIlluminanceMeasurement", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_occupancy_report: Fz.Converter<"msOccupancySensing", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_temperature_report: Fz.Converter<"msTemperatureMeasurement", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_humidity_report: Fz.Converter<"msRelativeHumidity", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_pressure_report: Fz.Converter<"msPressureMeasurement", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_analog_report: Fz.Converter<"genAnalogInput", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_multistate_report: Fz.Converter<"genMultistateInput", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_power_report: Fz.Converter<"genPowerCfg", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_light_brightness_report: Fz.Converter<"genLevelCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_light_color_colortemp_report: Fz.Converter<"lightingColorCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_closuresWindowCovering_report: Fz.Converter<"closuresWindowCovering", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_thermostat_report: Fz.Converter<"hvacThermostat", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_iaszone_attreport: Fz.Converter<"ssIasZone", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_iaszone_statuschange: Fz.Converter<"ssIasZone", undefined, "commandStatusChangeNotification">;
export declare const ignore_iaszone_report: Fz.Converter<"ssIasZone", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_iasace_commandgetpanelstatus: Fz.Converter<"ssIasAce", undefined, ["commandGetPanelStatus"]>;
export declare const ignore_genIdentify: Fz.Converter<"genIdentify", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_command_on: Fz.Converter<"genOnOff", undefined, "commandOn">;
export declare const ignore_command_off: Fz.Converter<"genOnOff", undefined, "commandOff">;
export declare const ignore_command_off_with_effect: Fz.Converter<"genOnOff", undefined, "commandOffWithEffect">;
export declare const ignore_command_step: Fz.Converter<"genLevelCtrl", undefined, "commandStep">;
export declare const ignore_command_stop: Fz.Converter<"genLevelCtrl", undefined, "commandStop">;
export declare const ignore_genLevelCtrl_report: Fz.Converter<"genLevelCtrl", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_haDiagnostic: Fz.Converter<"haDiagnostic", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_tuya_set_time: Fz.Converter<"manuSpecificTuya", undefined, ["commandMcuSyncTime"]>;
export declare const ignore_tuya_raw: Fz.Converter<"manuSpecificTuya", undefined, ["raw"]>;
export declare const ignore_metering: Fz.Converter<"seMetering", undefined, ["attributeReport", "readResponse"]>;
export declare const ignore_electrical_measurement: Fz.Converter<"haElectricalMeasurement", undefined, ["attributeReport", "readResponse"]>;
export declare const command_arm_with_transaction: Fz.Converter<"ssIasAce", undefined, "commandArm">;
export declare const ias_ace_occupancy_with_timeout: Fz.Converter<"ssIasAce", undefined, "commandGetPanelStatus">;
//# sourceMappingURL=fromZigbee.d.ts.map