import { Bytes, fixed_list, int8s, LVBytes, LVList, list, uint_t, uint8_t, uint16_t, uint24_t, uint32_t, uint64_t, WordList } from "./basic";
import { Bool, EmberApsOption, EmberBindingType, EmberConcentratorType, EmberConfigTxPowerMode, EmberCounterType, EmberCurrentSecurityBitmask, EmberDerivedKeyType, EmberDeviceUpdate, EmberEUI64, EmberEventUnits, EmberGpKeyType, EmberGpSecurityLevel, EmberIncomingMessageType, EmberInitialSecurityBitmask, EmberJoinDecision, EmberJoinMethod, EmberKeyStatus, EmberKeyStructBitmask, EmberKeyType, EmberLibraryStatus, EmberMacPassthroughType, EmberMulticastId, EmberNetworkInitBitmask, EmberNetworkStatus, EmberNodeId, EmberNodeType, EmberOutgoingMessageType, EmberPanId, EmberRf4ceApplicationCapabilities, EmberRf4ceNodeCapabilities, EmberRf4ceTxOption, EmberStackError, EmberStatus, EmberZDOCmd, EmberZdoConfigurationFlags, EmberZllKeyIndex, EmberZllState, EzspConfigId, EzspDecisionId, EzspEndpointFlags, EzspExtendedValueId, EzspMfgTokenId, EzspNetworkScanType, EzspPolicyId, EzspSourceRouteOverheadInformation, EzspStatus, EzspValueId, EzspZllNetworkOperation, NcpResetCode, SecureEzspRandomNumber, SecureEzspSecurityLevel, SecureEzspSecurityType, SecureEzspSessionId, SLStatus } from "./named";
import { EmberAesMmoHashContext, EmberApsFrame, EmberBindingTableEntry, EmberCertificate283k1Data, EmberCertificateData, EmberCurrentSecurityState, EmberGpAddress, EmberGpSinkListEntry, EmberInitialSecurityState, EmberKeyData, EmberKeyStruct, EmberMessageDigest, EmberMultiAddress, EmberMulticastTableEntry, EmberNeighbors, EmberNeighborTableEntry, EmberNetworkInitStruct, EmberNetworkParameters, EmberNodeDescriptor, EmberPrivateKey283k1Data, EmberPrivateKeyData, EmberPublicKey283k1Data, EmberPublicKeyData, EmberRf4ceApplicationInfo, EmberRf4cePairingTableEntry, EmberRf4ceVendorInfo, EmberRouteTableEntry, EmberRoutingTable, EmberRoutingTableEntry, EmberSecurityManagerContext, EmberSecurityManagerNetworkKeyInfo, EmberSignature283k1Data, EmberSignatureData, EmberSimpleDescriptor, EmberSmacData, EmberTokTypeStackZllData, EmberTokTypeStackZllSecurity, EmberZigbeeNetwork, EmberZllAddressAssignment, EmberZllDeviceInfoRecord, EmberZllInitialSecurityState, EmberZllNetwork, EmberZllSecurityAlgorithmData, EzspStruct } from "./struct";
export declare function deserialize(payload: any, schema: any[]): any[];
export declare function serialize(data: any[], schema: {
    serialize: (schema: any, item: any) => Buffer;
}[]): Buffer;
export { Bool, Bytes, EmberAesMmoHashContext, EmberApsFrame, EmberApsOption, EmberBindingTableEntry, EmberBindingType, EmberCertificate283k1Data, EmberCertificateData, EmberConcentratorType, EmberConfigTxPowerMode, EmberCounterType, EmberCurrentSecurityBitmask, EmberCurrentSecurityState, EmberDerivedKeyType, EmberDeviceUpdate, EmberEUI64, EmberEventUnits, EmberGpAddress, EmberGpKeyType, EmberGpSecurityLevel, EmberGpSinkListEntry, EmberIncomingMessageType, EmberInitialSecurityBitmask, EmberInitialSecurityState, EmberJoinDecision, EmberJoinMethod, EmberKeyData, EmberKeyStatus, EmberKeyStruct, EmberKeyStructBitmask, EmberKeyType, EmberLibraryStatus, EmberMacPassthroughType, EmberMessageDigest, EmberMultiAddress, EmberMulticastId, EmberMulticastTableEntry, EmberNeighbors, EmberNeighborTableEntry, EmberNetworkInitBitmask, EmberNetworkInitStruct, EmberNetworkParameters, EmberNetworkStatus, EmberNodeDescriptor, EmberNodeId, EmberNodeType, EmberOutgoingMessageType, EmberPanId, EmberPrivateKey283k1Data, EmberPrivateKeyData, EmberPublicKey283k1Data, EmberPublicKeyData, EmberRf4ceApplicationCapabilities, EmberRf4ceApplicationInfo, EmberRf4ceNodeCapabilities, EmberRf4cePairingTableEntry, EmberRf4ceTxOption, EmberRf4ceVendorInfo, EmberRouteTableEntry, EmberRoutingTable, EmberRoutingTableEntry, EmberSecurityManagerContext, EmberSecurityManagerNetworkKeyInfo, EmberSignature283k1Data, EmberSignatureData, EmberSimpleDescriptor, EmberSmacData, EmberStackError, EmberStatus, EmberTokTypeStackZllData, EmberTokTypeStackZllSecurity, EmberZDOCmd, EmberZdoConfigurationFlags, EmberZigbeeNetwork, EmberZllAddressAssignment, EmberZllDeviceInfoRecord, EmberZllInitialSecurityState, EmberZllKeyIndex, EmberZllNetwork, EmberZllSecurityAlgorithmData, EmberZllState, EzspConfigId, EzspDecisionId, EzspEndpointFlags, EzspExtendedValueId, EzspMfgTokenId, EzspNetworkScanType, EzspPolicyId, EzspSourceRouteOverheadInformation, EzspStatus, EzspStruct, EzspValueId, EzspZllNetworkOperation, fixed_list, int8s, LVBytes, LVList, list, NcpResetCode, SecureEzspRandomNumber, SecureEzspSecurityLevel, SecureEzspSecurityType, SecureEzspSessionId, SLStatus, uint_t, uint8_t, uint16_t, uint24_t, uint32_t, uint64_t, WordList, };
//# sourceMappingURL=index.d.ts.map