// Code generated by Herebyfile.mjs generate:enums from internal/core/compileroptions.go. DO NOT EDIT.
export var ModuleDetectionKind;
(function (ModuleDetectionKind) {
    ModuleDetectionKind[ModuleDetectionKind["None"] = 0] = "None";
    ModuleDetectionKind[ModuleDetectionKind["Auto"] = 1] = "Auto";
    ModuleDetectionKind[ModuleDetectionKind["Legacy"] = 2] = "Legacy";
    ModuleDetectionKind[ModuleDetectionKind["Force"] = 3] = "Force";
})(ModuleDetectionKind || (ModuleDetectionKind = {}));
//# sourceMappingURL=moduleDetectionKind.js.map