const noColor = /* @__PURE__ */ (() => {
	const proc = globalThis.process;
	const env = proc?.env ?? {};
	if (env.FORCE_COLOR) return false;
	if (env.NO_COLOR || env.TERM === "dumb") return true;
	return !proc?.stdout?.isTTY;
})();
const _c = (c, r = 39) => (t) => noColor ? t : `\u001B[${c}m${t}\u001B[${r}m`;
const bold = /* @__PURE__ */ _c(1, 22);
const red = /* @__PURE__ */ _c(31);
const green = /* @__PURE__ */ _c(32);
const yellow = /* @__PURE__ */ _c(33);
const blue = /* @__PURE__ */ _c(34);
const magenta = /* @__PURE__ */ _c(35);
const cyan = /* @__PURE__ */ _c(36);
const gray = /* @__PURE__ */ _c(90);
const url = (title, url) => noColor ? `[${title}](${url})` : `\u001B]8;;${url}\u001B\\${title}\u001B]8;;\u001B\\`;
export { blue, bold, cyan, gray, green, magenta, red, url, yellow };
