export default class TopicAliasRecv {
    private aliasToTopic;
    max: number;
    length: number;
    constructor(max: number);
    put(topic: string, alias: number): boolean;
    getTopicByAlias(alias: number): string;
    clear(): void;
}
