import { EmberInitialSecurityState } from "../types/struct";
import crc16ccitt from "./crc16ccitt";
declare function emberSecurity(networkKey: Buffer): EmberInitialSecurityState;
export { crc16ccitt, emberSecurity };
//# sourceMappingURL=index.d.ts.map