import { TResolveSetResponseResultPromise } from '../types';
export declare const createClearTimer: (clearTimeout: (typeof globalThis)["clearTimeout"], identifiersAndResolvers: Map<number, [number, TResolveSetResponseResultPromise]>) => (timerId: number) => Promise<boolean>;
//# sourceMappingURL=clear-timer.d.ts.map