import { SyntaxKind } from "../../ast/index.ts";
export declare const childProperties: Readonly<Partial<Record<SyntaxKind, readonly (string | undefined)[]>>>;
export declare const singleChildNodePropertyNames: Readonly<Partial<Record<SyntaxKind, string>>>;
//# sourceMappingURL=protocol.generated.d.ts.map