var o=(r=>(r[r.NeighborIsParent=0]="NeighborIsParent",r[r.NeighborIsAChild=1]="NeighborIsAChild",r[r.NeighborIsASibling=2]="NeighborIsASibling",r[r.NoneOfTheAbove=3]="NoneOfTheAbove",r[r.NeighborIsPreviousChild=4]="NeighborIsPreviousChild",r))(o||{});const I={Coordinator:"#ffff00",Router:"#0000ff",EndDevice:"#00ff00"},N={0:"#ff0000",1:"#44ff88",2:"#8888ff"};function s(r,f){return r.fillStyle=f,r.fillRect(0,0,1,1),`rgba(${r.getImageData(0,0,1,1).data.join(", ")});`}export{N as E,I as N,o as Z,s as c};
