import type { JsxEmit } from "#enums/jsxEmit";
import type { ModuleDetectionKind } from "#enums/moduleDetectionKind";
import type { ModuleKind } from "#enums/moduleKind";
import type { ModuleResolutionKind } from "#enums/moduleResolutionKind";
import type { NewLineKind } from "#enums/newLineKind";
import type { ScriptTarget } from "#enums/scriptTarget";
export interface CompilerOptions {
    allowJs?: boolean;
    allowArbitraryExtensions?: boolean;
    allowImportingTsExtensions?: boolean;
    allowNonTsExtensions?: boolean;
    allowUmdGlobalAccess?: boolean;
    allowUnreachableCode?: boolean;
    allowUnusedLabels?: boolean;
    assumeChangesOnlyAffectDirectDependencies?: boolean;
    checkJs?: boolean;
    customConditions?: string[];
    composite?: boolean;
    emitDeclarationOnly?: boolean;
    emitBOM?: boolean;
    emitDecoratorMetadata?: boolean;
    declaration?: boolean;
    declarationDir?: string;
    declarationMap?: boolean;
    deduplicatePackages?: boolean;
    disableSizeLimit?: boolean;
    disableSourceOfProjectReferenceRedirect?: boolean;
    disableSolutionSearching?: boolean;
    disableReferencedProjectLoad?: boolean;
    erasableSyntaxOnly?: boolean;
    exactOptionalPropertyTypes?: boolean;
    experimentalDecorators?: boolean;
    forceConsistentCasingInFileNames?: boolean;
    isolatedModules?: boolean;
    isolatedDeclarations?: boolean;
    ignoreConfig?: boolean;
    ignoreDeprecations?: string;
    importHelpers?: boolean;
    inlineSourceMap?: boolean;
    inlineSources?: boolean;
    init?: boolean;
    incremental?: boolean;
    jsx?: JsxEmit;
    jsxFactory?: string;
    jsxFragmentFactory?: string;
    jsxImportSource?: string;
    lib?: string[];
    libReplacement?: boolean;
    locale?: string;
    mapRoot?: string;
    module?: ModuleKind;
    moduleResolution?: ModuleResolutionKind;
    moduleSuffixes?: string[];
    moduleDetection?: ModuleDetectionKind;
    newLine?: NewLineKind;
    noEmit?: boolean;
    noCheck?: boolean;
    noErrorTruncation?: boolean;
    noFallthroughCasesInSwitch?: boolean;
    noImplicitAny?: boolean;
    noImplicitThis?: boolean;
    noImplicitReturns?: boolean;
    noEmitHelpers?: boolean;
    noLib?: boolean;
    noPropertyAccessFromIndexSignature?: boolean;
    noUncheckedIndexedAccess?: boolean;
    noEmitOnError?: boolean;
    noUnusedLocals?: boolean;
    noUnusedParameters?: boolean;
    noResolve?: boolean;
    noImplicitOverride?: boolean;
    noUncheckedSideEffectImports?: boolean;
    outDir?: string;
    paths?: Record<string, string[]>;
    preserveConstEnums?: boolean;
    preserveSymlinks?: boolean;
    project?: string;
    resolveJsonModule?: boolean;
    resolvePackageJsonExports?: boolean;
    resolvePackageJsonImports?: boolean;
    removeComments?: boolean;
    rewriteRelativeImportExtensions?: boolean;
    reactNamespace?: string;
    rootDir?: string;
    rootDirs?: string[];
    skipLibCheck?: boolean;
    stableTypeOrdering?: boolean;
    strict?: boolean;
    strictBindCallApply?: boolean;
    strictBuiltinIteratorReturn?: boolean;
    strictFunctionTypes?: boolean;
    strictNullChecks?: boolean;
    strictPropertyInitialization?: boolean;
    stripInternal?: boolean;
    skipDefaultLibCheck?: boolean;
    sourceMap?: boolean;
    sourceRoot?: string;
    suppressOutputPathCheck?: boolean;
    target?: ScriptTarget;
    traceResolution?: boolean;
    tsBuildInfoFile?: string;
    typeRoots?: string[];
    types?: string[];
    useDefineForClassFields?: boolean;
    useUnknownInCatchVariables?: boolean;
    verbatimModuleSyntax?: boolean;
    maxNodeModuleJsDepth?: number;
}
//# sourceMappingURL=compilerOptions.d.ts.map