export declare enum ModuleResolutionKind {
    Unknown = 0,
    Classic = 1,
    Node10 = 2,
    Node16 = 3,
    NodeNext = 99,
    Bundler = 100
}
//# sourceMappingURL=moduleResolutionKind.enum.d.ts.map