import{u as a,a as p,I as m,r as c,s as d,j as e,dp as f}from"./index-CE-9RlbR.js";import{I as l}from"./InfoAlert-AzL4ZLp4.js";import{S as g}from"./SettingsList-DXaUDCbO.js";import"./envs-CRUy-_iG.js";import"./CheckboxField-BPvsXluL.js";import"./InputField-DRmYNLGX.js";import"./NumberField-XGSCkVuy.js";import"./SelectField-BPXpeItQ.js";function k({sourceIdx:t,group:i}){const{t:r}=a(["settings","common"]),o=p(m(s=>s.bridgeInfo[t])),n=c.useCallback(async s=>{await d(t,"bridge/request/group/options",{id:i.id.toString(),options:s})},[t,i.id]);return e.jsxs(e.Fragment,{children:[e.jsx(l,{children:e.jsx("a",{href:f,target:"_blank",rel:"noreferrer",className:"link link-hover",children:r(s=>s.read_the_docs_info,{ns:"common"})})}),e.jsx(g,{schema:o.config_schema.definitions.group??{properties:{}},data:o.config.groups[i.id]??{},set:n,namespace:"definitions-group"})]})}export{k as default};
