import type { Adapter } from "../adapter";
export declare class Touchlink {
    private adapter;
    private locked;
    constructor(adapter: Adapter);
    stop(): Promise<void>;
    lock(lock: boolean): void;
    private transactionNumber;
    setChannelInterPAN(channel: number): Promise<void>;
    restoreChannelInterPAN(): Promise<void>;
    scan(): Promise<{
        ieeeAddr: string;
        channel: number;
    }[]>;
    identify(ieeeAddr: string, channel: number): Promise<void>;
    factoryReset(ieeeAddr: string, channel: number): Promise<boolean>;
    factoryResetFirst(): Promise<boolean>;
}
export default Touchlink;
//# sourceMappingURL=touchlink.d.ts.map