"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.definitions = void 0;
const zigbee_herdsman_1 = require("zigbee-herdsman");
const exposes = __importStar(require("../lib/exposes"));
const m = __importStar(require("../lib/modernExtend"));
const e = exposes.presets;
const MFR = 0x1994; // Gewiss S.p.A.
const gewissShutterConfigure = {
    configure: [
        async (device, coordinatorEndpoint) => {
            const endpoint = device.getEndpoint(1);
            if (endpoint) {
                try {
                    await endpoint.bind("closuresWindowCovering", coordinatorEndpoint);
                    await endpoint.configureReporting("closuresWindowCovering", [
                        {
                            attribute: "currentPositionLiftPercentage",
                            minimumReportInterval: 1,
                            maximumReportInterval: 600,
                            reportableChange: 1,
                        },
                    ]);
                }
                catch {
                    // Silently catch errors if device rejects other reporting parameters
                }
            }
        },
    ],
    isModernExtend: true,
};
exports.definitions = [
    {
        zigbeeModel: ["GWA1201_TWO_WAY_SWITCH"],
        model: "GWA1201",
        vendor: "Gewiss",
        description: "Chorus on/off switch",
        extend: [
            m.onOff({ powerOnBehavior: true }),
            m.identify(),
            m.deviceAddCustomCluster("gewissLed", {
                ID: 0xfd79,
                name: "gewissLed",
                manufacturerCode: MFR,
                attributes: {
                    ledStandby: { ID: 0x0001, type: zigbee_herdsman_1.Zcl.DataType.UINT8, name: "ledStandby" },
                },
                commands: {},
                commandsResponse: {},
            }),
            m.numeric({
                name: "led_standby_brightness",
                cluster: "gewissLed",
                attribute: { ID: 0x0001, type: zigbee_herdsman_1.Zcl.DataType.UINT8 },
                description: "LED standby brightness",
                valueMin: 0,
                valueMax: 100,
                valueStep: 1,
                unit: "%",
                scale: 255 / 100,
                access: "ALL",
                zigbeeCommandOptions: { manufacturerCode: MFR },
            }),
        ],
        ota: true,
    },
    {
        zigbeeModel: ["GWA1241_PUSH_BUTTON_2_CH"],
        model: "GWA1241",
        vendor: "Gewiss",
        description: "ChorusSmart 2-channel push button controller",
        endpoint: (device) => {
            return { top: 1, bottom: 10 };
        },
        fromZigbee: [
            {
                cluster: "genBinaryInput",
                type: ["attributeReport", "readResponse"],
                convert: (model, msg, publish, options, meta) => {
                    if (Object.hasOwn(msg.data, "presentValue")) {
                        if (msg.data["presentValue"] === 1 || msg.data["presentValue"] === true) {
                            const button = msg.endpoint.ID === 1 ? "top" : msg.endpoint.ID === 10 ? "bottom" : "unknown";
                            if (button !== "unknown") {
                                return { action: `press_${button}` };
                            }
                        }
                    }
                },
            },
        ],
        toZigbee: [],
        exposes: [e.action(["press_top", "press_bottom"])],
        meta: { multiEndpoint: true },
    },
    {
        zigbeeModel: ["GWA1521_Actuator_1_CH_PF"],
        model: "GWA1521",
        description: "Switch actuator 1 channel with input",
        vendor: "Gewiss",
        extend: [m.onOff()],
    },
    {
        zigbeeModel: ["GWA1522_Actuator_2_CH"],
        model: "GWA1522",
        description: "Switch actuator 2 channels with input",
        vendor: "Gewiss",
        extend: [m.deviceEndpoints({ endpoints: { l1: 1, l2: 2 } }), m.onOff({ endpointNames: ["l1", "l2"] })],
    },
    {
        zigbeeModel: ["GWA1531_Shutter", "GWA1231_SHUTTER"],
        model: "GWA1231",
        vendor: "Gewiss",
        description: "Chorus roller shutter module",
        whiteLabel: [{ model: "GWA1531", fingerprint: [{ modelID: "GWA1531_Shutter" }] }],
        extend: [
            m.windowCovering({ controls: ["lift"], coverInverted: true, configureReporting: false }),
            m.deviceAddCustomCluster("gewissLed", {
                ID: 0xfd79,
                name: "gewissLed",
                manufacturerCode: MFR,
                attributes: {
                    ledStandby: { ID: 0x0001, type: zigbee_herdsman_1.Zcl.DataType.UINT8, name: "ledStandby" },
                    ledMovimento: { ID: 0x0003, type: zigbee_herdsman_1.Zcl.DataType.UINT8, name: "ledMovimento" },
                },
                commands: {},
                commandsResponse: {},
            }),
            gewissShutterConfigure,
            m.numeric({
                name: "opening_time",
                cluster: "closuresWindowCovering",
                attribute: { ID: 0x0101, type: zigbee_herdsman_1.Zcl.DataType.UINT16 },
                description: "Opening/ascent time in seconds",
                valueMin: 0,
                valueMax: 300,
                valueStep: 1,
                unit: "s",
                access: "ALL",
                zigbeeCommandOptions: { manufacturerCode: MFR },
            }),
            m.numeric({
                name: "closing_time",
                cluster: "closuresWindowCovering",
                attribute: { ID: 0x0102, type: zigbee_herdsman_1.Zcl.DataType.UINT16 },
                description: "Closing/descent time in seconds",
                valueMin: 0,
                valueMax: 300,
                valueStep: 1,
                unit: "s",
                access: "ALL",
                zigbeeCommandOptions: { manufacturerCode: MFR },
            }),
            m.numeric({
                name: "led_standby_brightness",
                cluster: "gewissLed",
                attribute: { ID: 0x0001, type: zigbee_herdsman_1.Zcl.DataType.UINT8 },
                description: "LED standby brightness",
                valueMin: 0,
                valueMax: 100,
                valueStep: 1,
                unit: "%",
                scale: 255 / 100,
                access: "ALL",
                zigbeeCommandOptions: { manufacturerCode: MFR },
            }),
            m.numeric({
                name: "led_moving_brightness",
                cluster: "gewissLed",
                attribute: { ID: 0x0003, type: zigbee_herdsman_1.Zcl.DataType.UINT8 },
                description: "LED brightness while moving",
                valueMin: 0,
                valueMax: 100,
                valueStep: 1,
                unit: "%",
                scale: 255 / 100,
                access: "ALL",
                zigbeeCommandOptions: { manufacturerCode: MFR },
            }),
        ],
    },
    {
        zigbeeModel: ["GWA1502_BinaryInput230V"],
        model: "GWA1502",
        vendor: "Gewiss",
        description: "Contact interface - 2 channels - 230V",
        meta: { multiEndpoint: true },
        extend: [
            m.deviceEndpoints({ endpoints: { "1": 1, "2": 2 } }),
            m.binary({
                name: "input",
                cluster: "genBinaryInput",
                attribute: "presentValue",
                reporting: { min: "MIN", max: "1_HOUR", change: 1 },
                valueOn: ["ON", 1],
                valueOff: ["OFF", 0],
                description: "State of input I1",
                access: "STATE_GET",
                endpointName: "1",
            }),
            m.binary({
                name: "input",
                cluster: "genBinaryInput",
                attribute: "presentValue",
                reporting: { min: "MIN", max: "1_HOUR", change: 1 },
                valueOn: ["ON", 1],
                valueOff: ["OFF", 0],
                description: "State of input I2",
                access: "STATE_GET",
                endpointName: "2",
            }),
        ],
    },
    {
        zigbeeModel: ["GWA1501_BinaryInput_FC"],
        model: "GWA1501",
        vendor: "Gewiss",
        description: "Contact interface - 2 channels",
        meta: { multiEndpoint: true },
        extend: [
            m.deviceEndpoints({ endpoints: { "1": 1, "2": 2 } }),
            m.battery(),
            m.binary({
                name: "input",
                cluster: "genBinaryInput",
                attribute: "presentValue",
                reporting: { min: "MIN", max: "MAX", change: 1 },
                valueOn: ["ON", 1],
                valueOff: ["OFF", 0],
                description: "State of input I1",
                access: "STATE_GET",
                endpointName: "1",
            }),
            m.binary({
                name: "input",
                cluster: "genBinaryInput",
                attribute: "presentValue",
                reporting: { min: "MIN", max: "MAX", change: 1 },
                valueOn: ["ON", 1],
                valueOff: ["OFF", 0],
                description: "State of input I2",
                access: "STATE_GET",
                endpointName: "2",
            }),
        ],
    },
];
//# sourceMappingURL=gewiss.js.map