export declare enum OuterExpressionKinds {
    Parentheses = 1,
    TypeAssertions = 2,
    NonNullAssertions = 4,
    PartiallyEmittedExpressions = 8,
    ExpressionsWithTypeArguments = 16,
    Satisfies = 32,
    ExcludeJSDocTypeAssertion = 64,
    Assignments = 128,
    Comma = 256,
    Assertions = 38,
    All = 63,
    AllExceptAssertionsOrExpressionsWithTypeArguments = 9,
    ExpressionTypePassthrough = 385
}
//# sourceMappingURL=outerExpressionKinds.enum.d.ts.map