import{r as n,j as s,F as r}from"./index-CE-9RlbR.js";import{D as e,a as i}from"./DisplayValue-eLtXORWe.js";const x=n.memo(({value:a})=>{const[o,t]=i(a);return s.jsxs(s.Fragment,{children:[s.jsx(r,{icon:o,className:t})," ",s.jsx(e,{value:a,name:"linkquality"})]})});export{x as L};
