import type { DefinitionWithExtend, Tz } from "../lib/types";
export declare const tzLocal: {
    easycode_auto_relock: {
        key: string[];
        convertSet: (entity: import("zigbee-herdsman/dist/controller/model").Endpoint | import("zigbee-herdsman/dist/controller/model").Group, key: string, value: unknown, meta: Tz.Meta) => Promise<{
            state: {
                auto_relock: unknown;
            };
        }>;
    };
};
export declare const definitions: DefinitionWithExtend[];
//# sourceMappingURL=onesti.d.ts.map