const i="https://www.zigbee2mqtt.io/images/devices/",o="https://www.zigbee2mqtt.io/images/vendors/",r=n=>{const t=n.definition?.model.replace(/:|\s|\//g,"-"),e=t?[`${i}${t}.png`]:[];if(n.definition&&n.definition.source!=="native"){const s=n.definition.vendor.replace(/:|\s|\//g,"-");e.push(`${o}${s}.png`)}return e},a=n=>{for(const t in n.endpoints)if(n.endpoints[t].clusters.output.includes("genOta"))return!0;return!1};export{r as g,a as h};
