type DecodeOptions = Parameters<TextDecoder["decode"]>[1];
export declare class Wtf8Decoder extends TextDecoder {
    decode(input?: NodeJS.AllowSharedBufferSource, options?: DecodeOptions): string;
}
export {};
//# sourceMappingURL=wtf8.d.ts.map