export declare enum SyntaxKind {
    Unknown = 0,
    EndOfFile = 1,
    SingleLineCommentTrivia = 2,
    MultiLineCommentTrivia = 3,
    NewLineTrivia = 4,
    WhitespaceTrivia = 5,
    ConflictMarkerTrivia = 6,
    NonTextFileMarkerTrivia = 7,
    NumericLiteral = 8,
    BigIntLiteral = 9,
    StringLiteral = 10,
    JsxText = 11,
    JsxTextAllWhiteSpaces = 12,
    RegularExpressionLiteral = 13,
    NoSubstitutionTemplateLiteral = 14,
    TemplateHead = 15,
    TemplateMiddle = 16,
    TemplateTail = 17,
    OpenBraceToken = 18,
    CloseBraceToken = 19,
    OpenParenToken = 20,
    CloseParenToken = 21,
    OpenBracketToken = 22,
    CloseBracketToken = 23,
    DotToken = 24,
    DotDotDotToken = 25,
    SemicolonToken = 26,
    CommaToken = 27,
    QuestionDotToken = 28,
    LessThanToken = 29,
    LessThanSlashToken = 30,
    GreaterThanToken = 31,
    LessThanEqualsToken = 32,
    GreaterThanEqualsToken = 33,
    EqualsEqualsToken = 34,
    ExclamationEqualsToken = 35,
    EqualsEqualsEqualsToken = 36,
    ExclamationEqualsEqualsToken = 37,
    EqualsGreaterThanToken = 38,
    PlusToken = 39,
    MinusToken = 40,
    AsteriskToken = 41,
    AsteriskAsteriskToken = 42,
    SlashToken = 43,
    PercentToken = 44,
    PlusPlusToken = 45,
    MinusMinusToken = 46,
    LessThanLessThanToken = 47,
    GreaterThanGreaterThanToken = 48,
    GreaterThanGreaterThanGreaterThanToken = 49,
    AmpersandToken = 50,
    BarToken = 51,
    CaretToken = 52,
    ExclamationToken = 53,
    TildeToken = 54,
    AmpersandAmpersandToken = 55,
    BarBarToken = 56,
    QuestionToken = 57,
    ColonToken = 58,
    AtToken = 59,
    QuestionQuestionToken = 60,
    BacktickToken = 61,
    HashToken = 62,
    EqualsToken = 63,
    PlusEqualsToken = 64,
    MinusEqualsToken = 65,
    AsteriskEqualsToken = 66,
    AsteriskAsteriskEqualsToken = 67,
    SlashEqualsToken = 68,
    PercentEqualsToken = 69,
    LessThanLessThanEqualsToken = 70,
    GreaterThanGreaterThanEqualsToken = 71,
    GreaterThanGreaterThanGreaterThanEqualsToken = 72,
    AmpersandEqualsToken = 73,
    BarEqualsToken = 74,
    BarBarEqualsToken = 75,
    AmpersandAmpersandEqualsToken = 76,
    QuestionQuestionEqualsToken = 77,
    CaretEqualsToken = 78,
    Identifier = 79,
    PrivateIdentifier = 80,
    JSDocCommentTextToken = 81,
    BreakKeyword = 82,
    CaseKeyword = 83,
    CatchKeyword = 84,
    ClassKeyword = 85,
    ConstKeyword = 86,
    ContinueKeyword = 87,
    DebuggerKeyword = 88,
    DefaultKeyword = 89,
    DeleteKeyword = 90,
    DoKeyword = 91,
    ElseKeyword = 92,
    EnumKeyword = 93,
    ExportKeyword = 94,
    ExtendsKeyword = 95,
    FalseKeyword = 96,
    FinallyKeyword = 97,
    ForKeyword = 98,
    FunctionKeyword = 99,
    IfKeyword = 100,
    ImportKeyword = 101,
    InKeyword = 102,
    InstanceOfKeyword = 103,
    NewKeyword = 104,
    NullKeyword = 105,
    ReturnKeyword = 106,
    SuperKeyword = 107,
    SwitchKeyword = 108,
    ThisKeyword = 109,
    ThrowKeyword = 110,
    TrueKeyword = 111,
    TryKeyword = 112,
    TypeOfKeyword = 113,
    VarKeyword = 114,
    VoidKeyword = 115,
    WhileKeyword = 116,
    WithKeyword = 117,
    ImplementsKeyword = 118,
    InterfaceKeyword = 119,
    LetKeyword = 120,
    PackageKeyword = 121,
    PrivateKeyword = 122,
    ProtectedKeyword = 123,
    PublicKeyword = 124,
    StaticKeyword = 125,
    YieldKeyword = 126,
    AbstractKeyword = 127,
    AccessorKeyword = 128,
    AsKeyword = 129,
    AssertsKeyword = 130,
    AssertKeyword = 131,
    AnyKeyword = 132,
    AsyncKeyword = 133,
    AwaitKeyword = 134,
    BooleanKeyword = 135,
    ConstructorKeyword = 136,
    DeclareKeyword = 137,
    GetKeyword = 138,
    ImmediateKeyword = 139,
    InferKeyword = 140,
    IntrinsicKeyword = 141,
    IsKeyword = 142,
    KeyOfKeyword = 143,
    ModuleKeyword = 144,
    NamespaceKeyword = 145,
    NeverKeyword = 146,
    OutKeyword = 147,
    ReadonlyKeyword = 148,
    RequireKeyword = 149,
    NumberKeyword = 150,
    ObjectKeyword = 151,
    SatisfiesKeyword = 152,
    SetKeyword = 153,
    StringKeyword = 154,
    SymbolKeyword = 155,
    TypeKeyword = 156,
    UndefinedKeyword = 157,
    UniqueKeyword = 158,
    UnknownKeyword = 159,
    UsingKeyword = 160,
    FromKeyword = 161,
    GlobalKeyword = 162,
    BigIntKeyword = 163,
    OverrideKeyword = 164,
    OfKeyword = 165,
    DeferKeyword = 166,
    QualifiedName = 167,
    ComputedPropertyName = 168,
    TypeParameter = 169,
    Parameter = 170,
    Decorator = 171,
    PropertySignature = 172,
    PropertyDeclaration = 173,
    MethodSignature = 174,
    MethodDeclaration = 175,
    ClassStaticBlockDeclaration = 176,
    Constructor = 177,
    GetAccessor = 178,
    SetAccessor = 179,
    CallSignature = 180,
    ConstructSignature = 181,
    IndexSignature = 182,
    TypePredicate = 183,
    TypeReference = 184,
    FunctionType = 185,
    ConstructorType = 186,
    TypeQuery = 187,
    TypeLiteral = 188,
    ArrayType = 189,
    TupleType = 190,
    OptionalType = 191,
    RestType = 192,
    UnionType = 193,
    IntersectionType = 194,
    ConditionalType = 195,
    InferType = 196,
    ParenthesizedType = 197,
    ThisType = 198,
    TypeOperator = 199,
    IndexedAccessType = 200,
    MappedType = 201,
    LiteralType = 202,
    NamedTupleMember = 203,
    TemplateLiteralType = 204,
    TemplateLiteralTypeSpan = 205,
    ImportType = 206,
    ObjectBindingPattern = 207,
    ArrayBindingPattern = 208,
    BindingElement = 209,
    ArrayLiteralExpression = 210,
    ObjectLiteralExpression = 211,
    PropertyAccessExpression = 212,
    ElementAccessExpression = 213,
    CallExpression = 214,
    NewExpression = 215,
    TaggedTemplateExpression = 216,
    TypeAssertionExpression = 217,
    ParenthesizedExpression = 218,
    FunctionExpression = 219,
    ArrowFunction = 220,
    DeleteExpression = 221,
    TypeOfExpression = 222,
    VoidExpression = 223,
    AwaitExpression = 224,
    PrefixUnaryExpression = 225,
    PostfixUnaryExpression = 226,
    BinaryExpression = 227,
    ConditionalExpression = 228,
    TemplateExpression = 229,
    YieldExpression = 230,
    SpreadElement = 231,
    ClassExpression = 232,
    OmittedExpression = 233,
    ExpressionWithTypeArguments = 234,
    AsExpression = 235,
    NonNullExpression = 236,
    MetaProperty = 237,
    SyntheticExpression = 238,
    SatisfiesExpression = 239,
    TemplateSpan = 240,
    SemicolonClassElement = 241,
    Block = 242,
    EmptyStatement = 243,
    VariableStatement = 244,
    ExpressionStatement = 245,
    IfStatement = 246,
    DoStatement = 247,
    WhileStatement = 248,
    ForStatement = 249,
    ForInStatement = 250,
    ForOfStatement = 251,
    ContinueStatement = 252,
    BreakStatement = 253,
    ReturnStatement = 254,
    WithStatement = 255,
    SwitchStatement = 256,
    LabeledStatement = 257,
    ThrowStatement = 258,
    TryStatement = 259,
    DebuggerStatement = 260,
    VariableDeclaration = 261,
    VariableDeclarationList = 262,
    FunctionDeclaration = 263,
    ClassDeclaration = 264,
    InterfaceDeclaration = 265,
    TypeAliasDeclaration = 266,
    EnumDeclaration = 267,
    ModuleDeclaration = 268,
    ModuleBlock = 269,
    CaseBlock = 270,
    NamespaceExportDeclaration = 271,
    ImportEqualsDeclaration = 272,
    ImportDeclaration = 273,
    ImportClause = 274,
    NamespaceImport = 275,
    NamedImports = 276,
    ImportSpecifier = 277,
    ExportAssignment = 278,
    ExportDeclaration = 279,
    NamedExports = 280,
    NamespaceExport = 281,
    ExportSpecifier = 282,
    MissingDeclaration = 283,
    ExternalModuleReference = 284,
    JsxElement = 285,
    JsxSelfClosingElement = 286,
    JsxOpeningElement = 287,
    JsxClosingElement = 288,
    JsxFragment = 289,
    JsxOpeningFragment = 290,
    JsxClosingFragment = 291,
    JsxAttribute = 292,
    JsxAttributes = 293,
    JsxSpreadAttribute = 294,
    JsxExpression = 295,
    JsxNamespacedName = 296,
    CaseClause = 297,
    DefaultClause = 298,
    HeritageClause = 299,
    CatchClause = 300,
    ImportAttributes = 301,
    ImportAttribute = 302,
    PropertyAssignment = 303,
    ShorthandPropertyAssignment = 304,
    SpreadAssignment = 305,
    EnumMember = 306,
    SourceFile = 307,
    JSDocTypeExpression = 308,
    JSDocNameReference = 309,
    JSDocAllType = 310,
    JSDocNullableType = 311,
    JSDocNonNullableType = 312,
    JSDocOptionalType = 313,
    JSDocVariadicType = 314,
    JSDoc = 315,
    JSDocText = 316,
    JSDocTypeLiteral = 317,
    JSDocSignature = 318,
    JSDocLink = 319,
    JSDocLinkCode = 320,
    JSDocLinkPlain = 321,
    JSDocUnknownTag = 322,
    JSDocAugmentsTag = 323,
    JSDocImplementsTag = 324,
    JSDocDeprecatedTag = 325,
    JSDocPublicTag = 326,
    JSDocPrivateTag = 327,
    JSDocProtectedTag = 328,
    JSDocReadonlyTag = 329,
    JSDocOverrideTag = 330,
    JSDocCallbackTag = 331,
    JSDocOverloadTag = 332,
    JSDocParameterTag = 333,
    JSDocReturnTag = 334,
    JSDocThisTag = 335,
    JSDocTypeTag = 336,
    JSDocTemplateTag = 337,
    JSDocTypedefTag = 338,
    JSDocSeeTag = 339,
    JSDocPropertyTag = 340,
    JSDocThrowsTag = 341,
    JSDocSatisfiesTag = 342,
    JSDocImportTag = 343,
    SyntaxList = 344,
    JSTypeAliasDeclaration = 345,
    JSImportDeclaration = 346,
    NotEmittedStatement = 347,
    PartiallyEmittedExpression = 348,
    SyntheticReferenceExpression = 349,
    NotEmittedTypeElement = 350,
    Count = 351,
    FirstAssignment = 63,
    LastAssignment = 78,
    FirstCompoundAssignment = 64,
    LastCompoundAssignment = 78,
    FirstReservedWord = 82,
    LastReservedWord = 117,
    FirstKeyword = 82,
    LastKeyword = 166,
    FirstFutureReservedWord = 118,
    LastFutureReservedWord = 126,
    FirstTypeNode = 183,
    LastTypeNode = 206,
    FirstPunctuation = 18,
    LastPunctuation = 78,
    FirstToken = 0,
    LastToken = 166,
    FirstLiteralToken = 8,
    LastLiteralToken = 14,
    FirstTemplateToken = 14,
    LastTemplateToken = 17,
    FirstBinaryOperator = 29,
    LastBinaryOperator = 78,
    FirstStatement = 244,
    LastStatement = 260,
    FirstNode = 167,
    FirstJSDocNode = 308,
    LastJSDocNode = 343,
    FirstJSDocTagNode = 322,
    LastJSDocTagNode = 343,
    FirstContextualKeyword = 127,
    LastContextualKeyword = 166,
    LastUnaryOperator = 54,
    FirstTriviaToken = 2,
    LastTriviaToken = 6
}
//# sourceMappingURL=syntaxKind.enum.d.ts.map