import type { Tz } from "../lib/types";
export declare const on_off: Tz.Converter;
export declare const light_color: Tz.Converter;
export declare const light_colortemp: Tz.Converter;
export declare const read: Tz.Converter;
export declare const write: Tz.Converter;
export declare const command: Tz.Converter;
export declare const factory_reset: Tz.Converter;
export declare const identify: Tz.Converter;
export declare const zcl_command: Tz.Converter;
export declare const arm_mode: Tz.Converter;
export declare const battery_percentage_remaining: Tz.Converter;
export declare const battery_voltage: Tz.Converter;
export declare const power_on_behavior: Tz.Converter;
export declare const light_color_mode: Tz.Converter;
export declare const light_color_options: Tz.Converter;
export declare const lock: Tz.Converter;
export declare const lock_auto_relock_time: Tz.Converter;
export declare const lock_sound_volume: Tz.Converter;
export declare const pincode_lock: Tz.Converter;
export declare const lock_userstatus: Tz.Converter;
export declare const cover_via_brightness: Tz.Converter;
export declare const warning: Tz.Converter;
export declare const warning_simple: Tz.Converter;
export declare const squawk: Tz.Converter;
export declare const cover_state: Tz.Converter;
export declare const cover_position_tilt: Tz.Converter;
export declare const cover_mode: Tz.Converter;
export declare const occupancy_timeout: Tz.Converter;
export declare const level_config: Tz.Converter;
export declare const ballast_config: Tz.Converter;
export declare const light_brightness_step: Tz.Converter;
export declare const light_brightness_move: Tz.Converter;
export declare const light_colortemp_step: Tz.Converter;
export declare const light_colortemp_move: Tz.Converter;
export declare const light_color_and_colortemp_via_color: Tz.Converter;
export declare const light_hue_saturation_step: Tz.Converter;
export declare const light_hue_saturation_move: Tz.Converter;
export declare const light_onoff_brightness: Tz.Converter;
export declare const light_colortemp_startup: Tz.Converter;
export declare const light_color_colortemp: Tz.Converter;
export declare const effect: Tz.Converter;
export declare const thermostat_remote_sensing: Tz.Converter;
export declare const thermostat_weekly_schedule: Tz.Converter;
export declare const thermostat_system_mode: Tz.Converter;
export declare const thermostat_control_sequence_of_operation: Tz.Converter;
export declare const thermostat_programming_operation_mode: Tz.Converter;
export declare const thermostat_temperature_display_mode: Tz.Converter;
export declare const thermostat_keypad_lockout: Tz.Converter;
export declare const thermostat_temperature_setpoint_hold: Tz.Converter;
export declare const thermostat_temperature_setpoint_hold_duration: Tz.Converter;
export declare const fan_mode: Tz.Converter;
export declare const fan_speed: Tz.Converter;
export declare const thermostat_local_temperature: Tz.Converter;
export declare const thermostat_outdoor_temperature: Tz.Converter;
export declare const thermostat_local_temperature_calibration: Tz.Converter;
export declare const thermostat_occupancy: Tz.Converter;
export declare const thermostat_clear_weekly_schedule: Tz.Converter;
export declare const thermostat_pi_heating_demand: Tz.Converter;
export declare const thermostat_running_state: Tz.Converter;
export declare const thermostat_occupied_heating_setpoint: Tz.Converter;
export declare const thermostat_unoccupied_heating_setpoint: Tz.Converter;
export declare const thermostat_occupied_cooling_setpoint: Tz.Converter;
export declare const thermostat_unoccupied_cooling_setpoint: Tz.Converter;
export declare const thermostat_setpoint_raise_lower: Tz.Converter;
export declare const thermostat_relay_status_log: Tz.Converter;
export declare const thermostat_running_mode: Tz.Converter;
export declare const thermostat_min_heat_setpoint_limit: Tz.Converter;
export declare const thermostat_max_heat_setpoint_limit: Tz.Converter;
export declare const thermostat_min_cool_setpoint_limit: Tz.Converter;
export declare const thermostat_max_cool_setpoint_limit: Tz.Converter;
export declare const thermostat_ac_louver_position: Tz.Converter;
export declare const electrical_measurement_power: Tz.Converter;
export declare const electrical_measurement_power_phase_b: Tz.Converter;
export declare const electrical_measurement_power_phase_c: Tz.Converter;
export declare const metering_power: Tz.Converter;
export declare const metering_status: Tz.Converter;
export declare const metering_extended_status: Tz.Converter;
export declare const currentsummdelivered: Tz.Converter;
export declare const currenttier1summdelivered: Tz.Converter;
export declare const currenttier2summdelivered: Tz.Converter;
export declare const currenttier3summdelivered: Tz.Converter;
export declare const currenttier4summdelivered: Tz.Converter;
export declare const currentsummreceived: Tz.Converter;
export declare const frequency: Tz.Converter;
export declare const electrical_measurement_power_reactive: Tz.Converter;
export declare const powerfactor: Tz.Converter;
export declare const acvoltage: Tz.Converter;
export declare const acvoltage_phase_b: Tz.Converter;
export declare const acvoltage_phase_c: Tz.Converter;
export declare const accurrent: Tz.Converter;
export declare const accurrent_phase_b: Tz.Converter;
export declare const accurrent_phase_c: Tz.Converter;
export declare const accurrent_neutral: Tz.Converter;
export declare const dccurrent: Tz.Converter;
export declare const dcvoltage: Tz.Converter;
export declare const dcpower: Tz.Converter;
export declare const temperature: Tz.Converter;
export declare const humidity: Tz.Converter;
export declare const TYZB01_on_off: Tz.Converter;
export declare const power_source: Tz.Converter;
export declare const scene_store: Tz.Converter;
export declare const scene_recall: Tz.Converter;
export declare const scene_add: Tz.Converter;
export declare const scene_remove: Tz.Converter;
export declare const scene_remove_all: Tz.Converter;
export declare const scene_rename: Tz.Converter;
export declare const ignore_transition: Tz.Converter;
export declare const ignore_rate: Tz.Converter;
//# sourceMappingURL=toZigbee.d.ts.map