export declare enum NodeFlags {
    None = 0,
    Let = 1,
    Const = 2,
    Using = 4,
    Reparsed = 8,
    Synthesized = 16,
    OptionalChain = 32,
    ExportContext = 64,
    ContainsThis = 128,
    HasImplicitReturn = 256,
    HasExplicitReturn = 512,
    DisallowInContext = 1024,
    YieldContext = 2048,
    DecoratorContext = 4096,
    AwaitContext = 8192,
    DisallowConditionalTypesContext = 16384,
    ThisNodeHasError = 32768,
    JavaScriptFile = 65536,
    ThisNodeOrAnySubNodesHasError = 131072,
    HasAsyncFunctions = 262144,
    PossiblyContainsDynamicImport = 524288,
    PossiblyContainsImportMeta = 1048576,
    HasJSDoc = 2097152,
    JSDoc = 4194304,
    Ambient = 8388608,
    InWithStatement = 16777216,
    JsonFile = 33554432,
    PossiblyContainsDeprecatedTag = 67108864,
    Unreachable = 134217728,
    ReparserTransformedLiteral = 268435456,
    BlockScoped = 7,
    Constant = 6,
    AwaitUsing = 6,
    ReachabilityCheckFlags = 768,
    ReachabilityAndEmitFlags = 262912,
    ContextFlags = 25263104,
    TypeExcludesFlags = 10240,
    PermanentlySetIncrementalFlags = 1572864,
    IdentifierHasExtendedUnicodeEscape = 128,
    IdentifierIsInJSDocNamespace = 262144,
    NestedNamespace = 32
}
//# sourceMappingURL=nodeFlags.enum.d.ts.map