import type { createGenerateUniqueNumber } from './generate-unique-number';
export declare const createAddUniqueNumber: (generateUniqueNumber: ReturnType<typeof createGenerateUniqueNumber>) => (set: Set<number>) => number;
//# sourceMappingURL=add-unique-number.d.ts.map