import{u as l,j as e,aP as i,F as s,aQ as n,aR as t,aS as c,aT as f,aU as m,aV as o}from"./index-CE-9RlbR.js";import"./envs-CRUy-_iG.js";const h=[e.jsxs("div",{className:"flex flex-col gap-3 items-center",children:[e.jsx("a",{target:"_blank",rel:"noopener noreferrer",href:"https://www.buymeacoffee.com/Nerivec",className:"h-10",children:e.jsx("img",{alt:"Nerivec",src:"https://img.buymeacoffee.com/button-api/?text=Thanks Nerivec&emoji=☕&slug=Nerivec&button_colour=FFDD00&font_colour=000000&font_family=Arial&outline_colour=000000&coffee_colour=ffffff",className:"h-10"})}),e.jsxs("a",{target:"_blank",rel:"noopener noreferrer",className:"btn btn-secondary",href:"https://github.com/sponsors/Nerivec",children:[e.jsx(s,{icon:o}),"Sponsor Nerivec on Github"]})]},"Nerivec"),e.jsxs("div",{className:"flex flex-col gap-3 items-center",children:[e.jsx("a",{target:"_blank",rel:"noopener noreferrer",href:"https://www.buymeacoffee.com/koenkk",className:"h-10",children:e.jsx("img",{alt:"koenkk",src:"https://img.buymeacoffee.com/button-api/?text=Thanks Koenkk&emoji=☕&slug=koenkk&button_colour=FFDD00&font_colour=000000&font_family=Arial&outline_colour=000000&coffee_colour=ffffff",className:"h-10"})}),e.jsxs("a",{target:"_blank",rel:"noopener noreferrer",className:"btn btn-secondary",href:"https://github.com/sponsors/Koenkk",children:[e.jsx(s,{icon:o}),"Sponsor Koenkk on Github"]})]},"koenkk")];function N(){const{t:a}=l("contribute");return e.jsx("div",{className:"mb-3",children:e.jsxs("div",{className:"flex flex-col gap-6 items-center",children:[e.jsxs("div",{className:"flex flex-col gap-3 items-center",children:[a(r=>r.donation_text),h.sort(()=>Math.random()-.5)]}),e.jsx("h2",{className:"text-lg font-bold",children:a(r=>r.development)}),e.jsx("p",{children:a(r=>r.contributing)}),e.jsxs("div",{className:"flex flex-row flex-wrap gap-4 justify-center",children:[e.jsxs("a",{className:"link link-hover link-primary",target:"_blank",rel:"noopener noreferrer",href:i,children:["WindFront",e.jsx(s,{icon:n,size:"lg",className:"ms-1"})]}),e.jsxs("a",{className:"link link-hover link-primary",target:"_blank",rel:"noopener noreferrer",href:t,children:[a(r=>r.translation),e.jsx(s,{icon:n,size:"lg",className:"ms-1"})]})]}),e.jsxs("div",{className:"flex flex-row flex-wrap gap-4 justify-center",children:[e.jsxs("a",{className:"link link-hover link-primary",target:"_blank",rel:"noopener noreferrer",href:c,children:["Zigbee2MQTT",e.jsx(s,{icon:n,size:"lg",className:"ms-1"})]}),e.jsxs("a",{className:"link link-hover link-primary",target:"_blank",rel:"noopener noreferrer",href:f,children:["Zigbee Herdsman Converters",e.jsx(s,{icon:n,size:"lg",className:"ms-1"})]}),e.jsxs("a",{className:"link link-hover link-primary",target:"_blank",rel:"noopener noreferrer",href:m,children:["Zigbee Herdsman",e.jsx(s,{icon:n,size:"lg",className:"ms-1"})]})]})]})})}export{N as default};
