import { EventEmitter } from "node:events";
export declare const frameParserEvents: EventEmitter<any>;
declare function processFrame(frame: Uint8Array): void;
export default processFrame;
//# sourceMappingURL=frameParser.d.ts.map