import type { Node } from "../../ast/index.ts";
import { SyntaxKind } from "../../ast/index.ts";
export declare function getNodeDataType(kind: SyntaxKind): number;
export declare function getNodeCommonData(node: Node): number;
//# sourceMappingURL=encoder.generated.d.ts.map