import{r as c,u as i,z as l,j as t}from"./index-CE-9RlbR.js";import{T as m}from"./TimeAgo-D_O5nVVz.js";const x=c.memo(({lastSeen:a,config:n,fallback:o=null})=>{const{i18n:r}=i(),e=l(a,n),s=e?new Date(e):void 0;return s?t.jsx("span",{children:t.jsx(m,{datetime:s,locale:r.language})}):o});export{x as L};
