import type { DefinitionWithExtend } from "../lib/types";
export interface WiserDeviceInfo {
    attributes: {
        deviceInfo: string;
    };
    commands: never;
    commandResponses: never;
}
export declare const definitions: DefinitionWithExtend[];
//# sourceMappingURL=schneider_electric.d.ts.map