import{r as t,j as r}from"./index-CE-9RlbR.js";function n({indeterminate:e,...o}){const c=t.useRef(null);return t.useEffect(()=>{typeof e=="boolean"&&(c.current.indeterminate=!o.checked&&e)},[c,e]),r.jsx("input",{ref:c,type:"checkbox",className:"checkbox",...o})}export{n as I};
