export var RegularExpressionFlags;
(function (RegularExpressionFlags) {
    RegularExpressionFlags[RegularExpressionFlags["None"] = 0] = "None";
    RegularExpressionFlags[RegularExpressionFlags["HasIndices"] = 1] = "HasIndices";
    RegularExpressionFlags[RegularExpressionFlags["Global"] = 2] = "Global";
    RegularExpressionFlags[RegularExpressionFlags["IgnoreCase"] = 4] = "IgnoreCase";
    RegularExpressionFlags[RegularExpressionFlags["Multiline"] = 8] = "Multiline";
    RegularExpressionFlags[RegularExpressionFlags["DotAll"] = 16] = "DotAll";
    RegularExpressionFlags[RegularExpressionFlags["Unicode"] = 32] = "Unicode";
    RegularExpressionFlags[RegularExpressionFlags["UnicodeSets"] = 64] = "UnicodeSets";
    RegularExpressionFlags[RegularExpressionFlags["Sticky"] = 128] = "Sticky";
    RegularExpressionFlags[RegularExpressionFlags["AnyUnicodeMode"] = 96] = "AnyUnicodeMode";
})(RegularExpressionFlags || (RegularExpressionFlags = {}));
//# sourceMappingURL=regularExpressionFlags.enum.js.map