"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZpiObject = exports.Znp = void 0;
var znp_1 = require("./znp");
Object.defineProperty(exports, "Znp", { enumerable: true, get: function () { return znp_1.Znp; } });
var zpiObject_1 = require("./zpiObject");
Object.defineProperty(exports, "ZpiObject", { enumerable: true, get: function () { return zpiObject_1.ZpiObject; } });
//# sourceMappingURL=index.js.map