import type { MtCmd, MtCmdAreqZdo, MtCmdSreqZdo } from "./tstype";
export declare function isMtCmdAreqZdo(cmd: MtCmd): cmd is MtCmdAreqZdo;
export declare function isMtCmdSreqZdo(cmd: MtCmd): cmd is MtCmdSreqZdo;
//# sourceMappingURL=utils.d.ts.map