/** The endpoint where the Zigbee Device Object (ZDO) resides. */
export declare const ZDO_ENDPOINT = 0;
/** The profile ID used by the Zigbee Device Object (ZDO). */
export declare const ZDO_PROFILE_ID = 0;
/** ZDO messages start with a sequence number. */
export declare const ZDO_MESSAGE_OVERHEAD = 1;
export declare const MULTICAST_BINDING = 1;
export declare const UNICAST_BINDING = 3;
/** 64-bit challenge value used by CHALLENGE_REQUEST/CHALLENGE_RESPONSE clusters */
export declare const CHALLENGE_VALUE_SIZE = 8;
/** The 256-bit Curve 25519 public point. */
export declare const CURVE_PUBLIC_POINT_SIZE = 32;
//# sourceMappingURL=consts.d.ts.map