export declare enum ModifierFlags {
    None = 0,
    Public = 1,
    Private = 2,
    Protected = 4,
    Readonly = 8,
    Override = 16,
    Export = 32,
    Abstract = 64,
    Ambient = 128,
    Static = 256,
    Accessor = 512,
    Async = 1024,
    Default = 2048,
    Const = 4096,
    In = 8192,
    Out = 16384,
    Decorator = 32768,
    Deprecated = 65536,
    JSDocPublic = 8388608,
    JSDocPrivate = 16777216,
    JSDocProtected = 33554432,
    JSDocReadonly = 67108864,
    JSDocOverride = 134217728,
    HasComputedJSDocModifiers = 268435456,
    HasComputedFlags = 536870912,
    SyntacticOrJSDocModifiers = 31,
    SyntacticOnlyModifiers = 65504,
    SyntacticModifiers = 65535,
    JSDocCacheOnlyModifiers = 260046848,
    JSDocOnlyModifiers = 65536,
    NonCacheOnlyModifiers = 131071,
    AccessibilityModifier = 7,
    ParameterPropertyModifier = 31,
    NonPublicAccessibilityModifier = 6,
    TypeScriptModifier = 28895,
    ExportDefault = 2080,
    All = 131071,
    Modifier = 98303,
    JavaScript = 3872
}
//# sourceMappingURL=modifierFlags.enum.d.ts.map