export declare const PROTOCOL_VERSION = 5;
export declare const HEADER_OFFSET_METADATA = 0;
export declare const HEADER_OFFSET_HASH_LO0 = 4;
export declare const HEADER_OFFSET_HASH_LO1 = 8;
export declare const HEADER_OFFSET_HASH_HI0 = 12;
export declare const HEADER_OFFSET_HASH_HI1 = 16;
export declare const HEADER_OFFSET_PARSE_OPTIONS = 20;
export declare const HEADER_OFFSET_STRING_TABLE_OFFSETS = 24;
export declare const HEADER_OFFSET_STRING_TABLE = 28;
export declare const HEADER_OFFSET_EXTENDED_DATA = 32;
export declare const HEADER_OFFSET_STRUCTURED_DATA = 36;
export declare const HEADER_OFFSET_NODES = 40;
export declare const HEADER_SIZE = 44;
export declare const NODE_LEN = 28;
export declare const NODE_OFFSET_KIND = 0;
export declare const NODE_OFFSET_POS = 4;
export declare const NODE_OFFSET_END = 8;
export declare const NODE_OFFSET_NEXT = 12;
export declare const NODE_OFFSET_PARENT = 16;
export declare const NODE_OFFSET_DATA = 20;
export declare const NODE_OFFSET_FLAGS = 24;
export declare const KIND_NODE_LIST = 4294967295;
export declare const NODE_DATA_TYPE_CHILDREN = 0;
export declare const NODE_DATA_TYPE_STRING = 1073741824;
export declare const NODE_DATA_TYPE_EXTENDED = 2147483648;
export declare const NODE_STRING_INDEX_MASK = 16777215;
export declare const NODE_EXTENDED_DATA_MASK = 16777215;
export { childProperties, singleChildNodePropertyNames } from "./protocol.generated.ts";
//# sourceMappingURL=protocol.d.ts.map