import{r as i,j as s}from"./index-CE-9RlbR.js";const c=i.memo(e=>{const{label:l,detail:t,onChange:a,onBlur:n,...r}=e;return s.jsxs("fieldset",{className:"fieldset",children:[l&&s.jsxs("legend",{className:"fieldset-legend",children:[l,e.required?" *":""]}),s.jsx("input",{className:`input min-w-xs ${e.pattern||e.required?"validator":""}`,onChange:a,onBlur:n,...r}),t&&s.jsx("p",{className:"label",children:t})]})});export{c as I};
