import{r,j as e}from"./index-CE-9RlbR.js";const d=r.memo(s=>{const{label:a,detail:l,onChange:c,...t}=s;return e.jsxs("fieldset",{className:"fieldset",children:[a&&e.jsxs("legend",{className:"fieldset-legend",children:[a,s.required?" *":""]}),l?e.jsxs("label",{className:"label",children:[e.jsx("input",{className:`checkbox ${s.required?"validator":""}`,type:"checkbox",onChange:c,...t}),l]}):e.jsx("input",{className:`checkbox ${s.required?"validator":""}`,type:"checkbox",onChange:c,...t})]})});export{d as C};
