export declare enum ElementFlags {
    None = 0,
    Required = 1,
    Optional = 2,
    Rest = 4,
    Variadic = 8,
    Fixed = 3,
    Variable = 12,
    NonRequired = 14,
    NonRest = 11
}
//# sourceMappingURL=elementFlags.enum.d.ts.map