import * as exposes from "./exposes";
import type { Fz, Tz } from "./types";
interface ZosungIrTransmit {
    attributes: never;
    commands: {
        zosungSendIRCode00: {
            seq: number;
            length: number;
            unk1: number;
            unk2: number;
            unk3: number;
            cmd: number;
            unk4: number;
        };
        zosungSendIRCode01: {
            zero: number;
            seq: number;
            length: number;
            unk1: number;
            unk2: number;
            unk3: number;
            cmd: number;
            unk4: number;
        };
        zosungSendIRCode02: {
            seq: number;
            position: number;
            maxlen: number;
        };
        zosungSendIRCode03: {
            zero: number;
            seq: number;
            position: number;
            msgpart: Buffer;
            msgpartcrc: number;
        };
        zosungSendIRCode04: {
            zero0: number;
            seq: number;
            zero1: number;
        };
        zosungSendIRCode05: {
            seq: number;
            zero: number;
        };
    };
    commandResponses: {
        zosungSendIRCode03Resp: {
            zero: number;
            seq: number;
            position: number;
            msgpart: Buffer;
            msgpartcrc: number;
        };
        zosungSendIRCode05Resp: {
            seq: number;
            zero: number;
        };
    };
}
export declare const zosungExtend: {
    addZosungIRTransmitCluster: () => import("./types").ModernExtend;
    addZosungIRControlCluster: () => import("./types").ModernExtend;
};
export declare const fzZosung: {
    zosung_send_ir_code_01: {
        cluster: "zosungIRTransmit";
        type: ["commandZosungSendIRCode01"];
        convert: (model: import("./types").Definition, msg: Fz.Message<"zosungIRTransmit", ZosungIrTransmit, ["commandZosungSendIRCode01"]>, publish: import("./types").Publish, options: import("./types").KeyValue, meta: Fz.Meta) => void;
    };
    zosung_send_ir_code_02: {
        cluster: "zosungIRTransmit";
        type: ["commandZosungSendIRCode02"];
        convert: (model: import("./types").Definition, msg: Fz.Message<"zosungIRTransmit", ZosungIrTransmit, ["commandZosungSendIRCode02"]>, publish: import("./types").Publish, options: import("./types").KeyValue, meta: Fz.Meta) => Promise<void>;
    };
    zosung_send_ir_code_04: {
        cluster: "zosungIRTransmit";
        type: ["commandZosungSendIRCode04"];
        convert: (model: import("./types").Definition, msg: Fz.Message<"zosungIRTransmit", ZosungIrTransmit, ["commandZosungSendIRCode04"]>, publish: import("./types").Publish, options: import("./types").KeyValue, meta: Fz.Meta) => Promise<void>;
    };
    zosung_send_ir_code_00: {
        cluster: "zosungIRTransmit";
        type: ["commandZosungSendIRCode00"];
        convert: (model: import("./types").Definition, msg: Fz.Message<"zosungIRTransmit", ZosungIrTransmit, ["commandZosungSendIRCode00"]>, publish: import("./types").Publish, options: import("./types").KeyValue, meta: Fz.Meta) => Promise<void>;
    };
    zosung_send_ir_code_03: {
        cluster: "zosungIRTransmit";
        type: ["commandZosungSendIRCode03Resp"];
        convert: (model: import("./types").Definition, msg: Fz.Message<"zosungIRTransmit", ZosungIrTransmit, ["commandZosungSendIRCode03Resp"]>, publish: import("./types").Publish, options: import("./types").KeyValue, meta: Fz.Meta) => Promise<void>;
    };
    zosung_send_ir_code_05: {
        cluster: "zosungIRTransmit";
        type: ["commandZosungSendIRCode05Resp"];
        convert: (model: import("./types").Definition, msg: Fz.Message<"zosungIRTransmit", ZosungIrTransmit, ["commandZosungSendIRCode05Resp"]>, publish: import("./types").Publish, options: import("./types").KeyValue, meta: Fz.Meta) => Promise<{
            learned_ir_code: any;
            learned_ir_timings: {
                modulation: number;
                timings: number[];
                timestamp: number;
            };
        }>;
    };
};
export declare const tzZosung: {
    zosung_ir_code_to_send: {
        key: string[];
        convertSet: (entity: import("zigbee-herdsman/dist/controller/model").Endpoint | import("zigbee-herdsman/dist/controller/model").Group, key: string, value: unknown, meta: Tz.Meta) => Promise<void>;
    };
    zosung_learn_ir_code: {
        key: string[];
        convertSet: (entity: import("zigbee-herdsman/dist/controller/model").Endpoint | import("zigbee-herdsman/dist/controller/model").Group, key: string, value: unknown, meta: Tz.Meta) => Promise<void>;
    };
};
export declare const presetsZosung: {
    learn_ir_code: () => exposes.Binary;
    learned_ir_code: () => exposes.Text;
    learned_ir_timings: () => exposes.Text;
    ir_code_to_send: () => exposes.Text;
    ir_emitter: () => exposes.Text;
};
export {};
//# sourceMappingURL=zosung.d.ts.map