import{u as a,a as m,I as c,r as p,s as f,j as s,bd as l}from"./index-CE-9RlbR.js";import{m as d}from"./merge-CNyRwwjh.js";import{I as g}from"./InfoAlert-AzL4ZLp4.js";import{S as h}from"./SettingsList-DXaUDCbO.js";import"./envs-CRUy-_iG.js";import"./isObjectLike-BsEMHB57.js";import"./isObject--vsEa_js.js";import"./CheckboxField-BPvsXluL.js";import"./InputField-DRmYNLGX.js";import"./NumberField-XGSCkVuy.js";import"./SelectField-BPXpeItQ.js";function O({sourceIdx:t,ieeeAddress:i}){const{t:r}=a(["settings","common"]),o=m(c(e=>e.bridgeInfo[t])),n=p.useCallback(async e=>{await f(t,"bridge/request/device/options",{id:i,options:e})},[t,i]);return s.jsxs(s.Fragment,{children:[s.jsx(g,{children:s.jsx("a",{href:l,target:"_blank",rel:"noreferrer",className:"link link-hover",children:r(e=>e.read_the_docs_info,{ns:"common"})})}),s.jsx(h,{schema:o.config_schema.definitions.device??{properties:{}},data:d({},o.config.device_options,o.config.devices[i]),set:n,namespace:"definitions-device"})]})}export{O as default};
