export declare enum RegularExpressionFlags {
    None = 0,
    HasIndices = 1,
    Global = 2,
    IgnoreCase = 4,
    Multiline = 8,
    DotAll = 16,
    Unicode = 32,
    UnicodeSets = 64,
    Sticky = 128,
    AnyUnicodeMode = 96
}
//# sourceMappingURL=regularExpressionFlags.enum.d.ts.map