export declare enum TokenFlags {
    None = 0,
    PrecedingLineBreak = 1,
    PrecedingJSDocComment = 2,
    Unterminated = 4,
    ExtendedUnicodeEscape = 8,
    Scientific = 16,
    Octal = 32,
    HexSpecifier = 64,
    BinarySpecifier = 128,
    OctalSpecifier = 256,
    ContainsSeparator = 512,
    UnicodeEscape = 1024,
    ContainsInvalidEscape = 2048,
    HexEscape = 4096,
    ContainsLeadingZero = 8192,
    ContainsInvalidSeparator = 16384,
    PrecedingJSDocLeadingAsterisks = 32768,
    SingleQuote = 65536,
    PrecedingJSDocWithDeprecated = 131072,
    PrecedingJSDocWithSeeOrLink = 262144,
    BinaryOrOctalSpecifier = 384,
    WithSpecifier = 448,
    StringLiteralFlags = 72716,
    NumericLiteralFlags = 25584,
    TemplateLiteralLikeFlags = 7180,
    RegularExpressionLiteralFlags = 4,
    IsInvalid = 26656
}
//# sourceMappingURL=tokenFlags.enum.d.ts.map