"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.attributeKeyValue = attributeKeyValue;
exports.attributeList = attributeList;
const logger_1 = require("../../utils/logger");
const Zcl = __importStar(require("../../zspec/zcl"));
const NS = "zh:controller:zcl";
const LEGRAND_GROUP_MANUF_CODE = Zcl.ManufacturerCode.LEGRAND_GROUP;
// NOTE: `legrandWorkaround`:
//   Legrand devices fail to set the manufacturerSpecific flag and manufacturerCode in the frame header, despite using specific attributes.
//   This leads to incorrect reported attribute names.
//   Remap the attributes using the target device's manufacturer ID if the header is lacking the information.
function attributeKeyValue(frame, deviceManufacturerID, customClusters) {
    const payload = {};
    const legrandWorkaround = frame.header?.manufacturerCode === undefined && deviceManufacturerID === LEGRAND_GROUP_MANUF_CODE;
    const cluster = legrandWorkaround ? Zcl.Utils.getCluster(frame.cluster.name, deviceManufacturerID, customClusters) : frame.cluster;
    const manufacturerCode = legrandWorkaround ? deviceManufacturerID : frame.header.manufacturerCode;
    // TODO: remove this type once Zcl.Frame is typed
    for (const item of frame.payload) {
        // Per ZCL spec, non-success `readRsp` records carry no value; don't synthesize one.
        if ("status" in item && item.status !== Zcl.Status.SUCCESS) {
            continue;
        }
        const attribute = Zcl.Utils.getClusterAttribute(cluster, item.attrId, manufacturerCode);
        if (attribute) {
            try {
                const attrData = Zcl.Utils.processAttributePostRead(attribute, item.attrData);
                payload[attribute.name] = attrData;
            }
            catch (error) {
                logger_1.logger.debug(`Ignoring attribute ${attribute.name} from response: ${error}`, NS);
            }
        }
        else {
            payload[item.attrId] = item.attrData;
        }
    }
    return payload;
}
function attributeList(frame, deviceManufacturerID, customClusters) {
    const payload = [];
    const legrandWorkaround = frame.header?.manufacturerCode === undefined && deviceManufacturerID === LEGRAND_GROUP_MANUF_CODE;
    const cluster = legrandWorkaround ? Zcl.Utils.getCluster(frame.cluster.name, deviceManufacturerID, customClusters) : frame.cluster;
    const manufacturerCode = legrandWorkaround ? deviceManufacturerID : frame.header.manufacturerCode;
    // TODO: remove this type once Zcl.Frame is typed
    for (const item of frame.payload) {
        const attribute = Zcl.Utils.getClusterAttribute(cluster, item.attrId, manufacturerCode);
        payload.push(attribute?.name ?? item.attrId);
    }
    return payload;
}
//# sourceMappingURL=zclFrameConverter.js.map