declare class ZclTransactionSequenceNumber {
    private sequence;
    get current(): number;
    next(): number;
}
declare const _default: ZclTransactionSequenceNumber;
export default _default;
//# sourceMappingURL=zclTransactionSequenceNumber.d.ts.map