export declare enum ObjectFlags {
    None = 0,
    Class = 1,
    Interface = 2,
    Reference = 4,
    Tuple = 8,
    Anonymous = 16,
    Mapped = 32,
    Instantiated = 64,
    ObjectLiteral = 128,
    EvolvingArray = 256,
    ObjectLiteralPatternWithComputedProperties = 512,
    ReverseMapped = 1024,
    JsxAttributes = 2048,
    JSLiteral = 4096,
    FreshLiteral = 8192,
    ArrayLiteral = 16384,
    PrimitiveUnion = 32768,
    ContainsWideningType = 65536,
    ContainsObjectOrArrayLiteral = 131072,
    NonInferrableType = 262144,
    CouldContainTypeVariablesComputed = 524288,
    CouldContainTypeVariables = 1048576,
    MembersResolved = 2097152,
    ClassOrInterface = 3,
    RequiresWidening = 196608,
    PropagatingFlags = 458752,
    InstantiatedMapped = 96,
    InstantiationExpressionType = 16777216,
    SingleSignatureType = 33554432,
    ObjectTypeKindMask = 50332991,
    ContainsSpread = 4194304,
    ObjectRestType = 8388608,
    IsClassInstanceClone = 67108864,
    IdenticalBaseTypeCalculated = 134217728,
    IdenticalBaseTypeExists = 268435456,
    UnresolvedMembers = 536870912,
    FromTypeNode = 1073741824,
    IsGenericTypeComputed = 4194304,
    IsGenericObjectType = 8388608,
    IsGenericIndexType = 16777216,
    IsGenericType = 25165824,
    ContainsIntersections = 33554432,
    IsUnknownLikeUnionComputed = 67108864,
    IsUnknownLikeUnion = 134217728,
    IsNeverIntersectionComputed = 33554432,
    IsNeverIntersection = 67108864,
    IsConstrainedTypeVariable = 134217728
}
//# sourceMappingURL=objectFlags.enum.d.ts.map