export declare enum SignatureFlags {
    None = 0,
    HasRestParameter = 1,
    HasLiteralTypes = 2,
    Construct = 4,
    Abstract = 8,
    IsInnerCallChain = 16,
    IsOuterCallChain = 32,
    IsUntypedSignatureInJSFile = 64,
    IsNonInferrable = 128,
    IsSignatureCandidateForOverloadFailure = 256,
    PropagatingFlags = 335,
    CallChainFlags = 48
}
//# sourceMappingURL=signatureFlags.enum.d.ts.map