import { Buffalo } from "../../buffalo";
import { BuffaloZclDataType, DataType } from "./definition/enums";
import type { BuffaloZclOptions, StructuredSelector } from "./definition/tstype";
export declare class BuffaloZcl extends Buffalo {
    private readLengthUInt8;
    private readLengthUInt16;
    private writeOctetStr;
    private readOctetStr;
    private writeCharStr;
    private readCharStr;
    private writeLongOctetStr;
    private readLongOctetStr;
    private writeLongCharStr;
    private readLongCharStr;
    private writeArray;
    private readArray;
    private writeStruct;
    private readStruct;
    private writeToD;
    private readToD;
    private writeDate;
    private readDate;
    private writeListZoneInfo;
    private readListZoneInfo;
    private writeExtensionFieldSets;
    private readExtensionFieldSets;
    private writeListThermoTransitions;
    private readListThermoTransitions;
    private writeGpdFrame;
    private readGpdFrame;
    writeStructuredSelector(value: StructuredSelector): void;
    readStructuredSelector(): StructuredSelector;
    private writeListTuyaDataPointValues;
    private readListTuyaDataPointValues;
    private writeListMiboxerZones;
    private readListMiboxerZones;
    private writeBigEndianUInt24;
    private readBigEndianUInt24;
    private readMiStruct;
    write(type: DataType | BuffaloZclDataType, value: any, options: BuffaloZclOptions): void;
    read(type: DataType | BuffaloZclDataType, options: BuffaloZclOptions): any;
}
//# sourceMappingURL=buffaloZcl.d.ts.map