// Code generated by Herebyfile.mjs generate:enums from internal/core/compileroptions.go. DO NOT EDIT.
export var ModuleResolutionKind;
(function (ModuleResolutionKind) {
    ModuleResolutionKind[ModuleResolutionKind["Unknown"] = 0] = "Unknown";
    ModuleResolutionKind[ModuleResolutionKind["Classic"] = 1] = "Classic";
    ModuleResolutionKind[ModuleResolutionKind["Node10"] = 2] = "Node10";
    ModuleResolutionKind[ModuleResolutionKind["Node16"] = 3] = "Node16";
    ModuleResolutionKind[ModuleResolutionKind["NodeNext"] = 99] = "NodeNext";
    ModuleResolutionKind[ModuleResolutionKind["Bundler"] = 100] = "Bundler";
})(ModuleResolutionKind || (ModuleResolutionKind = {}));
//# sourceMappingURL=moduleResolutionKind.js.map